@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,6 +44,7 @@
 #include <rcsc/math_util.h>
 
 //#define DEBUG
+//#define DEBUG_NECK_TURN_TO_BALL
 
 namespace rcsc {
 
@@ -331,13 +332,13 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
     {
         dlog.addText( Logger::ACTION,
                       __FILE__": (Neck_TurnToBall) ball is near." );
-        //agent->doTurnNeck( ball_rel_angle_next - wm.self().neck() );
-        //return true;
         angle_buf = std::max( 0.0, next_view_width * 0.5 - 20.0 ); // 2008-07-11
     }
 
     if ( ( wm.ball().pos().x > 0.0
            || wm.ball().pos().absY() > ServerParam::i().pitchHalfWidth() - 8.0
+           || ! opp
+           || opp->distFromBall() > 3.0
            )
          && wm.opponentsFromSelf().size() >= 11 )
     {
@@ -349,19 +350,37 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
         int best_score = 0;
         double best_angle_diff = 0.0;
 
-        const double dir_step = std::max( 5.0, ( angle_buf * 2.0 ) / 10.0 );
+        const double dir_step = std::max( 5.0, ( angle_buf * 2.0 ) / 20.0 );
         const double max_dir = ball_angle_next.degree() + angle_buf + dir_step * 0.5;
 
         dlog.addText( Logger::ACTION,
-                      __FILE__": (Neck_TurnToBall) angle_buf=%.1f max_dir=%f dir_step=%f",
+                      __FILE__": (Neck_TurnToBall) angle_buf=%.1f max_dir=%.2f dir_step=%.1f",
                       angle_buf,
                       max_dir, dir_step );
+        dlog.addText( Logger::ACTION,
+                      __FILE__": (Neck_TurnToBall) ball_angle=%.1f start_dir=%.1f",
+                      ball_angle_next.degree(),
+                      ball_angle_next.degree() - angle_buf );
 
         const AbstractPlayerCont::const_iterator end = wm.allPlayers().end();
         for ( double dir = ball_angle_next.degree() - angle_buf;
               dir < max_dir;
               dir += dir_step )
         {
+#if 1
+            AngleDeg neck_angle = AngleDeg( dir ) - my_body_next;
+            if ( neck_angle.degree() > ServerParam::i().maxNeckAngle()
+                 || neck_angle.degree() < ServerParam::i().minNeckAngle() )
+            {
+#ifdef DEBUG_NECK_TURN_TO_BALL
+                dlog.addText( Logger::ACTION,
+                              "__ dir=%.0f over the next neck range",
+                              dir );
+#endif
+                continue;
+            }
+#endif
+
             AngleDeg left_angle = dir - next_view_width * 0.5;
             AngleDeg right_angle = dir + next_view_width * 0.5;
 
@@ -403,7 +422,7 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
                     }
                 }
             }
-#ifdef DEBUG
+#ifdef DEBUG_NECK_TURN_TO_BALL
             dlog.addText( Logger::ACTION,
                           "__ dir=%.0f  score=%d  angle_diff=%.1f",
                           dir,
@@ -415,7 +434,7 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
                 best_dir = dir;
                 best_score = score;
                 best_angle_diff = angle_diff;
-#ifdef DEBUG
+#ifdef DEBUG_NECK_TURN_TO_BALL
                 dlog.addText( Logger::ACTION,
                               "__ update best dir (1)" );
 #endif
@@ -427,7 +446,7 @@ Neck_TurnToBall::execute( PlayerAgent * agent )
                     best_dir = dir;
                     best_score = score;
                     best_angle_diff = angle_diff;
-#ifdef DEBUG
+#ifdef DEBUG_NECK_TURN_TO_BALL
                     dlog.addText( Logger::ACTION,
                                   "__ update best dir (2)" );
 #endif
@@ -687,7 +706,7 @@ Bhv_NeckBodyToBall::execute( PlayerAgent * agent )
             = agent->world().ball().pos()
             + agent->world().ball().vel();
 
-        return Bhv_NeckBodyToPoint( ball_next ).execute( agent );
+        return Bhv_NeckBodyToPoint( ball_next, M_angle_buf ).execute( agent );
     }
     else
     {
