@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,10 +35,12 @@
 #include <rcsc/geom/vector_2d.h>
 #include <rcsc/geom/circle_2d.h>
 #include <rcsc/geom/rect_2d.h>
+#include <rcsc/geom/sector_2d.h>
 #include <rcsc/geom/triangle_2d.h>
 
 #include <boost/cstdint.hpp>
 
+#include <string>
 #include <cstdio>
 
 namespace rcsc {
@@ -92,13 +94,27 @@ public:
     Line := <Time> <Level> <Type> <Content>
     Time := integer value
     Level := integer value
-    Type :=  T | p | l | c | t | r | m
+    Type :=  M | p | l | a | c | C | t | T | r | R | s | m
+        M : log message
+        p : point
+        l: line
+        a: arc
+        c: circle
+        C: filled circle
+        t: triangle
+        T: filled triangle
+        r: rectangle
+        R: filled rectangle
+        s: sector
+        m: message;
     Text := <Str>
     Point := <x:Real> <y:Real>[ <Color>]
     Line := <x1:Real> <y1:Real> <x2:Real> <y2:Real>[ <Color>]
+    Arc := <x:Real> <y:Real> <r:Real> <startAngle:Real> <spanAngle:Real>[ <Color>]
     Circle := <x:Real> <y:Real> <r:Real>[ <Color>]
     Triangle := <x1:Real> <y1:Real> <x2:Real> <y2:Real> <x3:Real> <y3:Real>[ <Color>]
     Rectangle := <leftX:Real> <topY:Real> <width:Real> <height:Real>[ <Color>]
+    Sector := <x:Real> <y:Real> <minR:Real> <maxR:Real> <startAngle:Real> <spanAngle:Real>[ <Color>]
     Message := <x:Real> <y:Real>[ (c <Color>)] <Str>
     **************************************************/
 
@@ -118,6 +134,8 @@ public:
     static const boost::int32_t MARK      = LEVEL_14; //!< log level definition alias
     static const boost::int32_t ROLE      = LEVEL_15; //!< log level definition alias
     static const boost::int32_t TEAM      = LEVEL_16; //!< log level definition alias
+    static const boost::int32_t ANALYZER  = LEVEL_17; //!< log level definition alias
+    static const boost::int32_t RECURSIVE = LEVEL_18; //!< log level definition alias
 
 private:
 
@@ -163,9 +181,9 @@ public:
 
     /*!
       \brief open file to record
-      \param file_path file path to open
+      \param filepath file path string
      */
-    void open( const char * file_path );
+    void open( const std::string & filepath );
 
     /*!
       \brief check if file is opend
@@ -176,14 +194,6 @@ public:
           return ( M_fout != NULL );
       }
 
-    /*!
-      \brief put message to file directry. no flush. Do NOT use this method by yourself.
-      \param cycle cycle information
-      \param msg message
-     */
-    void printDebugClientMessage( const long & cycle,
-                                  const std::string & msg );
-
     /*!
       \brief flush stored message
     */
@@ -240,7 +250,7 @@ public:
     void addPoint( const boost::int32_t level,
                    const double & x,
                    const double & y,
-                   const char r, const char g, const char b );
+                   const int r, const int g, const int b );
 
     /*!
       \brief add point info to buffer with cycle, level & message tag 'p'
@@ -252,7 +262,7 @@ public:
      */
     void addPoint( const boost::int32_t level,
                    const Vector2D & pos,
-                   const char r, const char g, const char b )
+                   const int r, const int g, const int b )
       {
           addPoint( level, pos.x, pos.y, r, g, b );
       }
@@ -304,7 +314,7 @@ public:
                   const double & y1,
                   const double & x2,
                   const double & y2,
-                  const char r, const char g, const char b );
+                  const int r, const int g, const int b );
 
     /*!
       \brief add line info to buffer with cycle, level & message tag 'l'
@@ -318,11 +328,95 @@ public:
     void addLine( const boost::int32_t level,
                   const Vector2D & start,
                   const Vector2D & end,
-                  const char r, const char g, const char b )
+                  const int r, const int g, const int b )
       {
           addLine( level, start.x, start.y, end.x, end.y, r, g, b );
       }
 
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param x center x of the circle
+      \param y center y of the circle
+      \param radius radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param color color name string
+      \param r green value
+      \param g green value
+      \param b blue value
+     */
+    void addArc( const boost::int32_t level,
+                 const double & x,
+                 const double & y,
+                 const double & radius,
+                 const AngleDeg & start_angle,
+                 const double & span_angle,
+                 const char * color = NULL );
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param center center of the circle
+      \param radius radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param color color name string
+      \param r green value
+      \param g green value
+      \param b blue value
+     */
+    void addArc( const boost::int32_t level,
+                 const Vector2D & center,
+                 const double & radius,
+                 const AngleDeg & start_angle,
+                 const double & span_angle,
+                 const char * color = NULL )
+      {
+          addArc( level, center.x, center.y, radius, start_angle, span_angle, color );
+      }
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param x center x of the circle
+      \param y center y of the circle
+      \param radius radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param r green value
+      \param g green value
+      \param b blue value
+     */
+    void addArc( const boost::int32_t level,
+                 const double & x,
+                 const double & y,
+                 const double & radius,
+                 const AngleDeg & start_angle,
+                 const double & span_angle,
+                 const int r, const int g, const int b );
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param center center of the circle
+      \param radius radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param r green value
+      \param g green value
+      \param b blue value
+     */
+    void addArc( const boost::int32_t level,
+                 const Vector2D & center,
+                 const double & radius,
+                 const AngleDeg & start_angle,
+                 const double & span_angle,
+                 const int r, const int g, const int b )
+      {
+          addArc( level, center.x, center.y, radius, start_angle, span_angle, r, g, b );
+      }
+
     /*!
       \brief add circle info to buffer with cycle, level & message tag 'c'
       \param level log level variable
@@ -330,12 +424,14 @@ public:
       \param y circle center point coordinate y
       \param radius circle radius
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const double & x,
                     const double & y,
                     const double & radius,
-                    const char * color = NULL );
+                    const char * color = NULL,
+                    const bool fill = false );
 
     /*!
       \brief add circle info to buffer with cycle, level & message tag 'c'
@@ -343,13 +439,15 @@ public:
       \param center circle center point coordinate
       \param radius circle radius
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const Vector2D & center,
                     const double & radius,
-                    const char * color = NULL )
+                    const char * color = NULL,
+                    const bool fill = false )
       {
-          addCircle( level, center.x, center.y, radius, color );
+          addCircle( level, center.x, center.y, radius, color, fill );
       }
 
     /*!
@@ -357,12 +455,14 @@ public:
       \param level log level variable
       \param circle circle object
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const Circle2D & circle,
-                    const char * color = NULL )
+                    const char * color = NULL,
+                    const bool fill = false )
       {
-          addCircle( level, circle.center().x, circle.center().y, circle.radius(), color );
+          addCircle( level, circle.center().x, circle.center().y, circle.radius(), color, fill );
       }
 
     /*!
@@ -374,12 +474,14 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const double & x,
                     const double & y,
                     const double & radius,
-                    const char r, const char g, const char b );
+                    const int r, const int g, const int b,
+                    const bool fill = false );
 
     /*!
       \brief add circle info to buffer with cycle, level & message tag 'c'
@@ -389,13 +491,15 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const Vector2D & center,
                     const double & radius,
-                    const char r, const char g, const char b )
+                    const int r, const int g, const int b,
+                    const bool fill = false )
       {
-          addCircle( level, center.x, center.y, radius, r, g, b );
+          addCircle( level, center.x, center.y, radius, r, g, b, fill );
       }
 
     /*!
@@ -405,14 +509,17 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addCircle( const boost::int32_t level,
                     const Circle2D & circle,
-                    const char r, const char g, const char b )
+                    const int r, const int g, const int b,
+                    const bool fill = false )
       {
           addCircle( level,
                      circle.center().x, circle.center().y, circle.radius(),
-                     r, g, b );
+                     r, g, b,
+                     fill );
       }
 
     /*!
@@ -425,6 +532,7 @@ public:
       \param x3 3rd vertex coordinate x
       \param y3 3rd vertex coordinate y
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const double & x1,
@@ -433,7 +541,8 @@ public:
                       const double & y2,
                       const double & x3,
                       const double & y3,
-                      const char * color = NULL );
+                      const char * color = NULL,
+                      const bool fill = false );
 
     /*!
       \brief add triangle info to buffer with cycle, level & message tag 't'
@@ -442,18 +551,21 @@ public:
       \param p2 2nd vertex coordinate
       \param p3 3rd vertex coordinate
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const Vector2D & p1,
                       const Vector2D & p2,
                       const Vector2D & p3,
-                      const char * color = NULL )
+                      const char * color = NULL,
+                      const bool fill = false )
       {
           addTriangle( level,
                        p1.x, p1.y,
                        p2.x, p2.y,
                        p3.x, p3.y,
-                       color );
+                       color,
+                       fill );
       }
 
     /*!
@@ -461,16 +573,19 @@ public:
       \param level log level variable
       \param tri triangle object
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const Triangle2D & tri,
-                      const char * color = NULL )
+                      const char * color = NULL,
+                      const bool fill = false )
       {
           addTriangle( level,
                        tri.a().x, tri.a().y,
                        tri.b().x, tri.b().y,
                        tri.c().x, tri.c().y,
-                       color );
+                       color,
+                       fill );
       }
 
     /*!
@@ -485,6 +600,7 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const double & x1,
@@ -493,7 +609,8 @@ public:
                       const double & y2,
                       const double & x3,
                       const double & y3,
-                      const char r, const char g, const char b );
+                      const int r, const int g, const int b,
+                      const bool fill = false );
 
     /*!
       \brief add triangle info to buffer with cycle, level & message tag 't'
@@ -504,18 +621,21 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const Vector2D & p1,
                       const Vector2D & p2,
                       const Vector2D & p3,
-                      const char r, const char g, const char b )
+                      const int r, const int g, const int b,
+                      const bool fill = false )
       {
           addTriangle( level,
                        p1.x, p1.y,
                        p2.x, p2.y,
                        p3.x, p3.y,
-                       r, g, b );
+                       r, g, b,
+                       fill );
       }
 
     /*!
@@ -525,16 +645,19 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addTriangle( const boost::int32_t level,
                       const Triangle2D & tri,
-                      const char r, const char g, const char b )
+                      const int r, const int g, const int b,
+                      const bool fill = false )
       {
           addTriangle( level,
                        tri.a().x, tri.a().y,
                        tri.b().x, tri.b().y,
                        tri.c().x, tri.c().y,
-                       r, g, b );
+                       r, g, b,
+                       fill );
       }
 
     /*!
@@ -545,28 +668,33 @@ public:
       \param length x range of the rectangle
       \param width y range of the rectangle
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addRect( const boost::int32_t level,
                   const double & left,
                   const double & top,
                   const double & length,
                   const double & width,
-                  const char * color = NULL );
+                  const char * color = NULL,
+                  const bool fill = false );
 
     /*!
       \brief add rect info to buffer with cycle, level & message tag 'r'
       \param level log level variable
       \param rect rectangle object
       \param color color name string
+      \param fill switch to select a circle is filled or not.
      */
     void addRect( const boost::int32_t level,
                   const Rect2D & rect,
-                  const char * color = NULL )
+                  const char * color = NULL,
+                  const bool fill = false )
       {
           addRect( level,
                    rect.left(), rect.top(),
                    rect.size().length(), rect.size().width(),
-                   color );
+                   color,
+                   fill );
       }
 
     /*!
@@ -579,13 +707,15 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addRect( const boost::int32_t level,
                   const double & left,
                   const double & top,
                   const double & length,
                   const double & width,
-                  const char r, const char g, const char b );
+                  const int r, const int g, const int b,
+                  const bool fill = false );
 
     /*!
       \brief add rect info to buffer with cycle, level & message tag 'r'
@@ -594,17 +724,148 @@ public:
       \param r red value
       \param g green value
       \param b blue value
+      \param fill switch to select a circle is filled or not.
      */
     void addRect( const boost::int32_t level,
                   const Rect2D & rect,
-                  const char r, const char g, const char b )
+                  const int r, const int g, const int b,
+                  const bool fill = false )
       {
           addRect( level,
                    rect.left(), rect.top(),
                    rect.size().length(), rect.size().width(),
-                   r, g, b );
+                   r, g, b,
+                   fill );
       }
 
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param x center x of the circle
+      \param y center y of the circle
+      \param min_radius min radius value of the circle
+      \param max_radius min radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param color color name string
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const double & x,
+                    const double & y,
+                    const double & min_radius,
+                    const double & max_radius,
+                    const AngleDeg & start_angle,
+                    const double & span_angle,
+                    const char * color = NULL,
+                    const bool fill = false );
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param center center of the circle
+      \param min_radius min radius value of the circle
+      \param max_radius min radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param color color name string
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const Vector2D & center,
+                    const double & min_radius,
+                    const double & max_radius,
+                    const AngleDeg & start_angle,
+                    const double & span_angle,
+                    const char * color = NULL,
+                    const bool fill = false )
+      {
+          addSector( level,
+                     center.x, center.y, min_radius, max_radius,
+                     start_angle, span_angle,
+                     color,
+                     fill );
+      }
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param x center x of the circle
+      \param y center y of the circle
+      \param min_radius min radius value of the circle
+      \param max_radius min radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param r green value
+      \param g green value
+      \param b blue value
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const double & x,
+                    const double & y,
+                    const double & min_radius,
+                    const double & max_radius,
+                    const AngleDeg & start_angle,
+                    const double & span_angle,
+                    const int r, const int g, const int b,
+                    const bool fill = false );
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param center center of the circle
+      \param min_radius min radius value of the circle
+      \param max_radius min radius value of the circle
+      \param start_angle start angle
+      \param span_angle span of the arc. direction is clockwise from start_angle
+      \param r green value
+      \param g green value
+      \param b blue value
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const Vector2D & center,
+                    const double & min_radius,
+                    const double & max_radius,
+                    const AngleDeg & start_angle,
+                    const double & span_angle,
+                    const int r, const int g, const int b,
+                    const bool fill = false )
+      {
+          addSector( level,
+                     center.x, center.y, min_radius, max_radius,
+                     start_angle, span_angle,
+                     r, g, b,
+                     fill );
+      }
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param sector sector object
+      \param color color name string
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const Sector2D & sector,
+                    const char * color = NULL,
+                    const bool fill = false );
+
+    /*!
+      \brief add arc info to the buffer. message tag 'a'
+      \param level log level value
+      \param sector sector object
+      \param r green value
+      \param g green value
+      \param b blue value
+      \param fill switch to select a sector is filled or not
+     */
+    void addSector( const boost::int32_t level,
+                    const Sector2D & sector,
+                    const int r, const int g, const int b,
+                    const bool fill = false );
+
     /*!
       \brief add message info to buffer with cycle, level & message tag 'm'
       \param level log level variable
@@ -650,7 +911,7 @@ public:
                      const double & x,
                      const double & y,
                      const char * msg,
-                     const char r, const char g, const char b );
+                     const int r, const int g, const int b );
 
     /*!
       \brief add message info to buffer with cycle, level & message tag 'm'
@@ -664,7 +925,7 @@ public:
     void addMessage( const boost::int32_t level,
                      const Vector2D & pos,
                      const char * msg,
-                     const char r, const char g, const char b )
+                     const int r, const int g, const int b )
       {
           addMessage( level,
                       pos.x, pos.y, msg,
