@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -306,7 +306,7 @@ GoalieMessageParser::parse( const int sender,
                             const GameTime & current )
 {
     // format:
-    //    "g<unum_pos:4>"
+    //    "g<pos_body:4>"
     // the length of message == 5
 
     if ( *msg != sheader() )
@@ -349,7 +349,7 @@ GoalieMessageParser::parse( const int sender,
     goalie_pos.y = ( ival % 400 ) * 0.1 - 20.0;
     ival /= 400;
 
-    goalie_pos.x = ( ival % 160 ) * 0.1 + ( 52.5 - 16.0 );
+    goalie_pos.x = ( ival % 160 ) * 0.1 + ( 53.0 - 16.0 );
 
     dlog.addText( Logger::SENSOR,
                   "GoalieMessageParser: success! goalie pos = (%.2f %.2f) body = %.1f",
@@ -915,11 +915,11 @@ OnePlayerMessageParser::parse( const int sender,
                                const GameTime & current )
 {
     // format:
-    //    "P<unum_pos_body:4>"
-    // the length of message == 5
+    //    "P<unum_pos:3>"
+    // the length of message == 4
 
-    // 21 * 105/0.4 * 68/0.4 * 360/12
-    // 28113750 < 4 characters(74^4=29986576)
+    // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
+    //  -> 22 * 168 * 109 = 402864
 
     if ( *msg != sheader() )
     {
@@ -951,35 +951,259 @@ OnePlayerMessageParser::parse( const int sender,
 
     int player_unum = Unum_Unknown;
     Vector2D player_pos;
-    AngleDeg player_body;
 
-    // 30=360/12
-    player_body = static_cast< double >( ( ival % 30 ) * 12 - 180 );
-    ival /= 30;
+    // 109 > 68/0.63 + 1
+    player_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
 
-    // 170=68/0.4
-    player_pos.y = ( ival % 170 ) * 0.4 - 34.0;
-    ival /= 170;
+    // 168 > 105/0.63 + 1
+    player_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
 
-    // 263>105/0.4=262.5
-    player_pos.x = ( ival % 263 ) * 0.4 - 52.5;
-    ival /= 263;
-
-    player_unum = ( ival % 21 ) + 1;
-    // ival /= 21
+    // 22
+    player_unum = ( ival % 22 ) + 1;
+    ival /= 22;
 
     dlog.addText( Logger::SENSOR,
                   "OnePlayerMessageParser: success! "
-                  "unum = %d  pos(%.1f %.1f) body %.1f",
+                  "unum = %d  pos(%.1f %.1f)",
                   player_unum,
-                  player_pos.x, player_pos.y,
-                  player_body.degree() );
+                  player_pos.x, player_pos.y );
 
     M_memory->setPlayer( sender,
                          player_unum,
                          player_pos,
-                         player_body,
-                         -1.0, // unknown stamina
+                         current );
+
+    return slength();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+TwoPlayerMessageParser::TwoPlayerMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+TwoPlayerMessageParser::parse( const int sender,
+                               const double & ,
+                               const char * msg,
+                               const GameTime & current )
+{
+    // format:
+    //    "Q<unum_pos:3,unum_pos3>"
+    // the length of message == 7
+
+    // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
+    //  -> 22 * 168 * 109 = 402864
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( std::strlen( msg ) < slength() )
+    {
+        std::cerr << "TwoPlayerMessageParser::parse()"
+                  << " Illegal message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "TwoPlayerMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "TwoPlayerMessageParser: Failed to decode Player Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    int player1_unum = Unum_Unknown;
+    Vector2D player1_pos;
+    int player2_unum = Unum_Unknown;
+    Vector2D player2_pos;
+
+    // 109 > 68/0.63
+    player2_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
+
+    // 168 > 105/0.63
+    player2_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
+
+    // 22
+    player2_unum = ( ival % 22 ) + 1;
+    ival /= 22;
+
+
+    // 109 > 68/0.63
+    player1_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
+
+    // 168 > 105/0.63
+    player1_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
+
+    // 22
+    player1_unum = ( ival % 22 ) + 1;
+    ival /= 22;
+
+    dlog.addText( Logger::SENSOR,
+                  "TwoPlayerMessageParser: success! "
+                  "(unum=%d (%.2f %.2f)), (unum=%d (%.2f %.2f)) ",
+                  player1_unum,
+                  player1_pos.x, player1_pos.y,
+                  player2_unum,
+                  player2_pos.x, player2_pos.y );
+
+    M_memory->setPlayer( sender,
+                         player1_unum,
+                         player1_pos,
+                         current );
+    M_memory->setPlayer( sender,
+                         player2_unum,
+                         player2_pos,
+                         current );
+
+    return slength();
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+ThreePlayerMessageParser::ThreePlayerMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+ThreePlayerMessageParser::parse( const int sender,
+                                 const double & ,
+                                 const char * msg,
+                                 const GameTime & current )
+{
+    // format:
+    //    "R<unum_pos:3,unum_pos:3,unm_pos:3>"
+    // the length of message == 10
+
+    // ( 22 * 105/0.63 * 68/0.63 ) = 395767.195767196 < 74^3(=405224)
+    //  -> 22 * 168 * 109 = 402864
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( std::strlen( msg ) < slength() )
+    {
+        std::cerr << "ThreePlayerMessageParser::parse()"
+                  << " Illegal message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "ThreePlayerMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "ThreePlayerMessageParser: Failed to decode Player Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    int player1_unum = Unum_Unknown;
+    Vector2D player1_pos;
+    int player2_unum = Unum_Unknown;
+    Vector2D player2_pos;
+    int player3_unum = Unum_Unknown;
+    Vector2D player3_pos;
+
+    // 109 > 68/0.63
+    player3_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
+
+    // 168 > 105/0.63
+    player3_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
+
+    // 22
+    player3_unum = ( ival % 22 ) + 1;
+    ival /= 22;
+
+    // 109 > 68/0.63
+    player2_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
+
+    // 168 > 105/0.63
+    player2_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
+
+    // 22
+    player2_unum = ( ival % 22 ) + 1;
+    ival /= 22;
+
+
+    // 109 > 68/0.63
+    player1_pos.y = ( ival % 109 ) * 0.63 - 34.0;
+    ival /= 109;
+
+    // 168 > 105/0.63
+    player1_pos.x = ( ival % 168 ) * 0.63 - 52.5;
+    ival /= 168;
+
+    // 22
+    player1_unum = ( ival % 22 ) + 1;
+    ival /= 22;
+
+    dlog.addText( Logger::SENSOR,
+                  "ThreePlayerMessageParser: success! "
+                  "(unum=%d (%.2f %.2f)), (unum=%d (%.2f %.2f)), (unum=%d (%.2f %.2f)) ",
+                  player1_unum,
+                  player1_pos.x, player1_pos.y,
+                  player2_unum,
+                  player2_pos.x, player2_pos.y,
+                  player3_unum,
+                  player3_pos.x, player3_pos.y );
+
+    M_memory->setPlayer( sender,
+                         player1_unum,
+                         player1_pos,
+                         current );
+    M_memory->setPlayer( sender,
+                         player2_unum,
+                         player2_pos,
+                         current );
+    M_memory->setPlayer( sender,
+                         player3_unum,
+                         player3_pos,
                          current );
 
     return slength();
@@ -1042,22 +1266,21 @@ SelfMessageParser::parse( const int sender,
     AngleDeg player_body;
     double stamina = -1.0;
 
-    // 22
-    stamina = ServerParam::i().staminaMax()
-        * static_cast< double >( ( ival % 22 ) ) / 22.0;
-    ival /= 22;
+    // 11
+    stamina = ServerParam::i().staminaMax() * static_cast< double >( ( ival % 11 ) ) / 10.0;
+    ival /= 11;
 
-    // 30=360/12
-    player_body = static_cast< double >( ( ival % 30 ) * 12 - 180 );
-    ival /= 30;
+    // 60=360/6
+    player_body = ( ival % 60 ) * 6.0 - 180.0;
+    ival /= 60;
 
-    // 170=68/0.4
-    player_pos.y = ( ival % 170 ) * 0.4 - 34.0;
-    ival /= 170;
+    // 171 > 68/0.4
+    player_pos.y = ( ival % 171 ) * 0.4 - 34.0;
+    ival /= 171;
 
-    // 263>105/0.4=262.5
-    player_pos.x = ( ival % 263 ) * 0.4 - 52.5;
-    //ival /= 263;
+    // 264 > 105/0.4=262.5
+    player_pos.x = ( ival % 264 ) * 0.4 - 52.5;
+    //ival /= 264;
 
     dlog.addText( Logger::SENSOR,
                   "SelfMessageParser: success! "
@@ -1070,7 +1293,7 @@ SelfMessageParser::parse( const int sender,
     M_memory->setPlayer( sender,
                          player_unum,
                          player_pos,
-                         player_body,
+                         player_body.degree(),
                          stamina,
                          current );
 
@@ -1080,6 +1303,193 @@ SelfMessageParser::parse( const int sender,
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+TeammateMessageParser::TeammateMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+TeammateMessageParser::parse( const int sender,
+                              const double & ,
+                              const char * msg,
+                              const GameTime & current )
+{
+    // format:
+    //    "T<unum_pos_body:4>"
+    // the length of message == 5
+
+    //11 * 105/0.7 * 68/0.7 * 360/2
+    // -> 11 * 151 * 98 * 180
+    // =29300040 < 4 characters(74^4=29986576)
+
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( std::strlen( msg ) < slength() )
+    {
+        std::cerr << "TeammateMessageParser::parse()"
+                  << " Illegal message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "TeammateMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "TeammateMessageParser: Failed to decode Player Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    int player_unum = Unum_Unknown;
+    Vector2D player_pos;
+    AngleDeg player_body;
+
+    // 180=360/2
+    player_body = static_cast< double >( ( ival % 180 ) * 2 - 180 );
+    ival /= 180;
+
+    // 98=68/0.7=97.14
+    player_pos.y = ( ival % 98 ) * 0.7 - 34.0;
+    ival /= 98;
+
+    // 151>105/0.7=150
+    player_pos.x = ( ival % 151 ) * 0.7 - 52.5;
+    ival /= 151;
+
+    player_unum = ( ival % 11 ) + 1;
+    // ival /= 11
+
+    dlog.addText( Logger::SENSOR,
+                  "TeammateMessageParser: success! "
+                  "unum = %d  pos(%.1f %.1f) body %.1f",
+                  player_unum,
+                  player_pos.x, player_pos.y,
+                  player_body.degree() );
+
+    M_memory->setPlayer( sender,
+                         player_unum,
+                         player_pos,
+                         player_body.degree(),
+                         -1.0, // unknown stamina
+                         current );
+
+    return slength();
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+OpponentMessageParser::OpponentMessageParser( boost::shared_ptr< AudioMemory > memory )
+    : M_memory( memory )
+{
+
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+OpponentMessageParser::parse( const int sender,
+                              const double & ,
+                              const char * msg,
+                              const GameTime & current )
+{
+    // format:
+    //    "O<unum_pos_body:4>"
+    // the length of message == 5
+
+    //11 * 105/0.7 * 68/0.7 * 360/2
+    // -> 11 * 151 * 98 * 180
+    // =29300040 < 4 characters(74^4=29986576)
+
+
+    if ( *msg != sheader() )
+    {
+        return 0;
+    }
+
+    if ( std::strlen( msg ) < slength() )
+    {
+        std::cerr << "OpponentMessageParser::parse()"
+                  << " Illegal message ["
+                  << msg << "] len = " << std::strlen( msg )
+                  << std::endl;
+        return -1;
+    }
+    ++msg;
+
+    boost::int64_t ival = 0;
+    if ( ! AudioCodec::i().decodeStrToInt64( std::string( msg, slength() - 1 ),
+                                             &ival ) )
+    {
+        std::cerr << "OpponentMessageParser::parse()"
+                  << " Failed to parse [" << msg << "]"
+                  << std::endl;
+        dlog.addText( Logger::SENSOR,
+                      "OpponentMessageParser: Failed to decode Player Info [%s]",
+                      msg );
+        return -1;
+    }
+
+    int player_unum = Unum_Unknown;
+    Vector2D player_pos;
+    AngleDeg player_body;
+
+    // 180=360/2
+    player_body = static_cast< double >( ( ival % 180 ) * 2 - 180 );
+    ival /= 180;
+
+    // 98=68/0.7=97.14
+    player_pos.y = ( ival % 98 ) * 0.7 - 34.0;
+    ival /= 98;
+
+    // 151>105/0.7=150
+    player_pos.x = ( ival % 151 ) * 0.7 - 52.5;
+    ival /= 151;
+
+    player_unum = ( ival % 11 ) + 1;
+    // ival /= 11
+
+    dlog.addText( Logger::SENSOR,
+                  "OpponentMessageParser: success! "
+                  "unum = %d  pos(%.1f %.1f) body %.1f",
+                  player_unum,
+                  player_pos.x, player_pos.y,
+                  player_body.degree() );
+
+    M_memory->setPlayer( sender,
+                         player_unum + 11,
+                         player_pos,
+                         player_body.degree(),
+                         -1.0, // unknown stamina
+                         current );
+
+    return slength();
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 BallPlayerMessageParser::BallPlayerMessageParser( boost::shared_ptr< AudioMemory > memory )
     : M_memory( memory )
@@ -1150,25 +1560,25 @@ BallPlayerMessageParser::parse( const int sender,
     Vector2D player_pos;
     AngleDeg player_body;
 
-    // 30=360/12
-    player_body = static_cast< double >( ( ival % 30 ) * 12 - 180 );
-    ival /= 30;
+    // 180=360/2
+    player_body = static_cast< double >( ( ival % 180 ) * 2 - 180 );
+    ival /= 180;
 
-    // 170=68/0.4
-    player_pos.y = ( ival % 170 ) * 0.4 - 34.0;
-    ival /= 170;
+    // 69 > 68/1.0
+    player_pos.y = ( ival % 69 ) * 1.0 - 34.0;
+    ival /= 69;
 
-    // 263>105/0.4=262.5
-    player_pos.x = ( ival % 263 ) * 0.4 - 52.5;
-    ival /= 263;
+    // 106 > 105/1.0
+    player_pos.x = ( ival % 106 ) * 1.0 - 52.5;
+    ival /= 106;
 
-    player_unum = ( ival % 21 ) + 1;
-    // ival /= 21
+    player_unum = ( ival % 22 ) + 1;
+    // ival /= 22
 
     dlog.addText( Logger::SENSOR,
                   "BallPlayerMessageParser: success! "
                   " bpos(%.1f %.1f) bvel(%.1f %.1f)"
-                  " unum = %d  pos(%.1f %.1f) body %.1f",
+                  " unum=%d  pos(%.1f %.1f) body %.1f",
                   ball_pos.x, ball_pos.y,
                   ball_vel.x, ball_vel.y,
                   player_unum,
@@ -1179,7 +1589,7 @@ BallPlayerMessageParser::parse( const int sender,
     M_memory->setPlayer( sender,
                          player_unum,
                          player_pos,
-                         player_body,
+                         player_body.degree(),
                          -1.0, // unknown stamina
                          current );
 
