@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_bpn.cpp
-	\brief BPN formation data classes Source File.
+  \file formation_bpn.cpp
+  \brief BPN formation data classes Source File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,6 +44,10 @@
 
 namespace rcsc {
 
+using namespace formation;
+
+const std::string FormationBPN::NAME( "BPN" );
+
 const double FormationBPN::Param::PITCH_LENGTH = 105.0 + 10.0;
 const double FormationBPN::Param::PITCH_WIDTH = 68.0 + 10.0;
 
@@ -53,7 +57,7 @@ const double FormationBPN::Param::PITCH_WIDTH = 68.0 + 10.0;
 
 */
 FormationBPN::Param::Param()
-    : M_param( 0.3, 0.9 )
+    : M_net( 0.3, 0.9 )
 {
 
 }
@@ -70,7 +74,7 @@ FormationBPN::Param::randomize()
     boost::variate_generator< boost::mt19937 &, boost::uniform_real<> >
         rng( gen, dst );
 
-    M_param.randomize( rng );
+    M_net.randomize( rng );
 }
 
 /*-------------------------------------------------------------------*/
@@ -82,10 +86,10 @@ FormationBPN::Param::getPosition( const Vector2D & ball_pos,
                                   const Formation::SideType type ) const
 {
     double msign = 1.0;
-    if ( type == Formation::SYNMETRY ) msign =  -1.0;
+    if ( type == Formation::SYMMETRY ) msign =  -1.0;
     if ( type == Formation::CENTER && ball_pos.y > 0.0 ) msign = -1.0;
 
-    PosNet::input_array input;
+    Net::input_array input;
 
     input[0] = std::max( 0.0,
                          std::min( ball_pos.x / PITCH_LENGTH + 0.5,
@@ -94,9 +98,9 @@ FormationBPN::Param::getPosition( const Vector2D & ball_pos,
                          std::min( (ball_pos.y * msign) / PITCH_WIDTH + 0.5,
                                    1.0 ) );
 
-    PosNet::output_array output;
+    Net::output_array output;
 
-    M_param.propagate( input, output );
+    M_net.propagate( input, output );
     //std::cerr << "getPosition. raw output = "
     //<< output[0] << " ,  " << output[1]
     //<< std::endl;
@@ -151,7 +155,7 @@ FormationBPN::Param::readRoleName( std::istream & is )
 
 */
 bool
-FormationBPN::Param::readParam( std::istream & is )
+FormationBPN::Param::readNet( std::istream & is )
 {
     std::string line_buf;
     if ( ! std::getline( is, line_buf ) )
@@ -169,7 +173,7 @@ FormationBPN::Param::readParam( std::istream & is )
         return false;
     }
 
-    if ( ! M_param.read( istr ) )
+    if ( ! M_net.read( istr ) )
     {
         std::cerr  << __FILE__ << ":" << __LINE__
                    << " Failed to read position param" << std::endl;
@@ -193,7 +197,7 @@ FormationBPN::Param::read( std::istream & is )
         return false;
     }
 
-    if ( ! readParam( is ) )
+    if ( ! readNet( is ) )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " Failed to read parameters" << std::endl;
@@ -226,9 +230,9 @@ FormationBPN::Param::printRoleName( std::ostream & os ) const
 
 */
 std::ostream &
-FormationBPN::Param::printParam( std::ostream & os ) const
+FormationBPN::Param::printNet( std::ostream & os ) const
 {
-    M_param.print( os ) << '\n';
+    M_net.print( os ) << '\n';
     return os;
 }
 
@@ -245,7 +249,7 @@ std::ostream &
 FormationBPN::Param::print( std::ostream & os ) const
 {
     printRoleName( os );
-    printParam( os );
+    printNet( os );
 
     return os << std::flush;
 }
@@ -268,10 +272,14 @@ FormationBPN::FormationBPN()
 /*!
 
 */
-Formation::Snapshot
-FormationBPN::createDefaultParam()
+void
+FormationBPN::createDefaultData()
 {
-#if 1
+    if ( ! M_samples )
+    {
+        M_samples = SampleDataSet::Ptr( new SampleDataSet() );
+    }
+
     // 1: goalie
     // 2: left center back
     // 3(2): right center back
@@ -285,56 +293,32 @@ FormationBPN::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    setSynmetryType( 3, 2 );
+    setSymmetryType( 3, 2, "CenterBack" );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    setSynmetryType( 5, 4 );
+    setSymmetryType( 5, 4, "SideBack" );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    setSynmetryType( 8, 7 );
+    setSymmetryType( 8, 7, "OffensiveHalf" );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    setSynmetryType( 10, 9 );
+    setSymmetryType( 10, 9, "SideForward" );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    data.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 0.0 ) );
 
-    return snap;
-#else
-    // 1: goalie
-    // 2: center back
-    // 3: left side back
-    // 4(3): right side back
-    // 5: left defensive half
-    // 6(5): right defensive half
-    // 7: offensive half
-    // 8: left side half
-    // 9(8): right side half
-    // 10: left forward
-    // 11(10): right forward
-    createNewRole( 1, "Goalie", Formation::CENTER );
-    createNewRole( 2, "CenterBack", Formation::CENTER );
-    createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
-    createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setSynmetryType( 6, 5 );
-    createNewRole( 7, "SideHalf", Formation::SIDE );
-    setSynmetryType( 8, 7 );
-    createNewRole( 9, "OffensiveHalf", Formation::CENTER );
-    createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
-#endif
+    M_samples->addData( *this, data, false );
 }
 
 
@@ -367,7 +351,7 @@ FormationBPN::setRoleName( const int unum,
 std::string
 FormationBPN::getRoleName( const int unum ) const
 {
-    const boost::shared_ptr< const FormationBPN::Param > p = param( unum );
+    const boost::shared_ptr< const FormationBPN::Param > p = getParam( unum );
     if ( ! p )
     {
         std::cerr << __FILE__ << ":" << __LINE__
@@ -408,7 +392,7 @@ FormationBPN::createNewRole( const int unum,
     else
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** You cannot create a new parameter for synmetry type."
+                  << " *** ERROR *** You cannot create a new parameter for symmetry type."
                   << std::endl;
         return;
     }
@@ -436,7 +420,7 @@ Vector2D
 FormationBPN::getPosition( const int unum,
                            const Vector2D & ball_pos ) const
 {
-    const boost::shared_ptr< const FormationBPN::Param > ptr = param( unum );
+    const boost::shared_ptr< const FormationBPN::Param > ptr = getParam( unum );
     if ( ! ptr )
     {
         std::cerr << __FILE__ << ':' << __LINE__
@@ -447,8 +431,8 @@ FormationBPN::getPosition( const int unum,
     }
 
     Formation::SideType type = Formation::SIDE;
-    if ( M_synmetry_number[unum - 1] > 0 )  type = Formation::SYNMETRY;
-    if ( M_synmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
+    if ( M_symmetry_number[unum - 1] > 0 )  type = Formation::SYMMETRY;
+    if ( M_symmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
 
     return ptr->getPosition( ball_pos, type );
 }
@@ -474,9 +458,10 @@ FormationBPN::getPositions( const Vector2D & focus_point,
 
 */
 void
-FormationBPN::train( const std::list< Snapshot > & train_data )
+FormationBPN::train()
 {
-    if ( train_data.empty() )
+    if ( ! M_samples
+         || M_samples->dataCont().empty() )
     {
         return;
     }
@@ -490,9 +475,9 @@ FormationBPN::train( const std::list< Snapshot > & train_data )
     {
         int number = unum;
         Formation::SideType type = Formation::SIDE;
-        if ( isSynmetryType( unum ) )
+        if ( isSymmetryType( unum ) )
         {
-            std::cerr << "  Training. player " << unum << " >>> synmetry type "
+            std::cerr << "  Training. player " << unum << " >>> symmetry type "
                       << std::endl;
             continue;
         }
@@ -518,13 +503,12 @@ FormationBPN::train( const std::list< Snapshot > & train_data )
             break;
         }
 
-        FormationBPN::Param::PosNet & net = param->getParam();
+        FormationBPN::Param::Net & net = param->net();
 
-        FormationBPN::Param::PosNet::input_array input;
-        FormationBPN::Param::PosNet::output_array teacher;
+        FormationBPN::Param::Net::input_array input;
+        FormationBPN::Param::Net::output_array teacher;
 
-        const std::list< Formation::Snapshot >::const_iterator snap_end
-            = train_data.end();
+        const SampleDataSet::DataCont::const_iterator data_end = M_samples->dataCont().end();
         int loop = 0;
         double ave_err = 0.0;
         double max_err = 0.0;
@@ -533,14 +517,13 @@ FormationBPN::train( const std::list< Snapshot > & train_data )
         {
             ave_err = 0.0;
             max_err = 0.0;
-            double snap_count = 1.0;
-            for ( std::list< Formation::Snapshot >::const_iterator snap
-                      = train_data.begin();
-                  snap != snap_end;
-                  ++snap, snap_count += 1.0 )
+            double data_count = 1.0;
+            for ( SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
+                  d != data_end;
+                  ++d, data_count += 1.0 )
             {
-                double by = snap->ball_.y;
-                double py = snap->players_[unum - 1].y;
+                double by = d->ball_.y;
+                double py = d->players_[unum - 1].y;
 
                 if ( type == Formation::CENTER
                      && by > 0.0 )
@@ -556,13 +539,13 @@ FormationBPN::train( const std::list< Snapshot > & train_data )
                 }
 
                 input[0] = min_max( 0.0,
-                                    snap->ball_.x / PITCH_LENGTH + 0.5,
+                                    d->ball_.x / PITCH_LENGTH + 0.5,
                                     1.0 );
                 input[1] = min_max( 0.0,
                                     by / PITCH_WIDTH + 0.5,
                                     1.0 );
                 teacher[0] = min_max( 0.0,
-                                      snap->players_[unum - 1].x / PITCH_LENGTH + 0.5,
+                                      d->players_[unum - 1].x / PITCH_LENGTH + 0.5,
                                       1.0 );
                 teacher[1] = std::max( 0.0,
                                        std::min( py / PITCH_WIDTH + 0.5, 1.0 ) );
@@ -573,8 +556,8 @@ FormationBPN::train( const std::list< Snapshot > & train_data )
                     max_err = err;
                 }
                 ave_err
-                    = ave_err * ( ( snap_count - 1.0 ) / snap_count )
-                    + err / snap_count;
+                    = ave_err * ( ( data_count - 1.0 ) / data_count )
+                    + err / data_count;
             }
 #if 0
             if ( loop % 500 == 0 )
@@ -622,7 +605,7 @@ FormationBPN::getParam( const int unum )
             ( static_cast< FormationBPN::Param * >( 0 ) );
     }
 
-    if ( M_synmetry_number[unum - 1] > 0 )
+    if ( M_symmetry_number[unum - 1] > 0 )
     {
         return boost::shared_ptr< FormationBPN::Param >
             ( static_cast< FormationBPN::Param * >( 0 ) );
@@ -634,7 +617,7 @@ FormationBPN::getParam( const int unum )
     if ( it == M_param_map.end() )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** Parameter not found! unum = "
+                  << " *** ERROR *** Neteter not found! unum = "
                   << unum << std::endl;
         return boost::shared_ptr< FormationBPN::Param >
             ( static_cast< FormationBPN::Param * >( 0 ) );
@@ -648,7 +631,7 @@ FormationBPN::getParam( const int unum )
 
 */
 boost::shared_ptr< const FormationBPN::Param >
-FormationBPN::param( const int unum ) const
+FormationBPN::getParam( const int unum ) const
 {
     if ( unum < 1 || 11 < unum )
     {
@@ -659,9 +642,9 @@ FormationBPN::param( const int unum ) const
             ( static_cast< FormationBPN::Param * >( 0 ) );
     }
 
-    const int player_number = ( M_synmetry_number[unum - 1] <= 0 // center or original
+    const int player_number = ( M_symmetry_number[unum - 1] <= 0 // center or original
                                 ? unum
-                                : M_synmetry_number[unum - 1] );
+                                : M_symmetry_number[unum - 1] );
 
     std::map< int, boost::shared_ptr< FormationBPN::Param > >::const_iterator
         it = M_param_map.find( player_number );
@@ -669,7 +652,7 @@ FormationBPN::param( const int unum ) const
     if ( it == M_param_map.end() )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** Parameter not found! unum = "
+                  << " *** ERROR *** Neteter not found! unum = "
                   << unum << std::endl;
         return boost::shared_ptr< const FormationBPN::Param >
             ( static_cast< FormationBPN::Param * >( 0 ) );
@@ -683,42 +666,13 @@ FormationBPN::param( const int unum ) const
 
 */
 bool
-FormationBPN::read( std::istream & is )
+FormationBPN::readConf( std::istream & is )
 {
-    int n_line = 0;
-
-    n_line += readName( is );
-    if ( n_line < 0 )
-    {
-        return false;
-    }
-
-    //---------------------------------------------------
-    // read each player's parameter set
     if ( ! readPlayers( is ) )
     {
         return false;
     }
 
-    //---------------------------------------------------
-    // check synmetry number circuration reference
-    for ( int i = 0; i < 11; ++i )
-    {
-        int refered_unum = M_synmetry_number[i];
-        if ( refered_unum <= 0 ) continue;
-        if ( M_synmetry_number[refered_unum - 1] > 0 )
-        {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** failed to read formation."
-                      << " Bad synmetrying. player "
-                      << i + 1
-                      << " mirro = " << refered_unum
-                      << " is already synmetrying player"
-                      << std::endl;
-            return false;
-        }
-    }
-
     return true;
 }
 
@@ -743,15 +697,15 @@ FormationBPN::readPlayers( std::istream & is )
 
         // check id
         int unum = 0;
-        int synmetry = 0;
+        int symmetry = 0;
         int n_read = 0;
         if ( std::sscanf( line_buf.c_str(),
                           " player %d %d %n ",
-                          &unum, &synmetry,
+                          &unum, &symmetry,
                           &n_read ) != 2
              || n_read == 0 )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__<< ':'
                       << " *** ERROR *** failed to read formation at number "
                       << i + 1
                       << " [" << line_buf << "]"
@@ -760,40 +714,40 @@ FormationBPN::readPlayers( std::istream & is )
         }
         if ( unum != i + 1 )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__<< ':'
                       << " *** ERROR *** failed to read formation "
                       << " Invalid player number.  Expected " << i + 1
                       << "  but read number = " << unum
                       << std::endl;
             return false;
         }
-        if ( synmetry == unum )
+        if ( symmetry == unum )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__<< ':'
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid synmetry number. at "
+                      << " Invalid symmetry number. at "
                       << i
                       << " mirroing player itself. unum = " << unum
-                      << "  synmetry = " << synmetry
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
-        if ( 11 < synmetry )
+        if ( 11 < symmetry )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid synmetry number. at "
+                      << " Invalid symmetry number. at "
                       << i
-                      << "  Synmetry number is out of range. unum = " << unum
-                      << "  synmetry = " << synmetry
+                      << "  Symmetry number is out of range. unum = " << unum
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
 
-        M_synmetry_number[i] = synmetry;
+        M_symmetry_number[i] = symmetry;
 
-        // this player is synmetry type
-        if ( synmetry > 0 )
+        // this player is symmetry type
+        if ( symmetry > 0 )
         {
             ++player_read;
             continue;
@@ -803,7 +757,7 @@ FormationBPN::readPlayers( std::istream & is )
         boost::shared_ptr< FormationBPN::Param > param( new FormationBPN::Param );
         if ( ! param->read( is ) )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
                       << " *** ERROR *** failed to read formation. at number "
                       << i + 1
                       << " [" << line_buf << "]"
@@ -817,47 +771,48 @@ FormationBPN::readPlayers( std::istream & is )
 
     if ( player_read != 11 )
     {
-        std::cerr << __FILE__ << ':' << __LINE__
+        std::cerr << __FILE__ << ':' << __LINE__ << ':'
                   << " ***ERROR*** Invalid formation format."
                   << " The number of read player is " << player_read
                   << std::endl;
         return false;
     }
 
+    if ( ! std::getline( is, line_buf )
+         || line_buf != "End" )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                  << " ***ERROR*** Illegal end tag."
+                  << std::endl;
+        return false;
+    }
+
     return true;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  condition <playmode> <>
-  player <unum> 0 <Role> <Param1> <Param2> ...
-  player <unum> <mirro> <Role>
-  ...
-  important <x> <y>
-  important <x> <y>
-  ...
+
 */
 std::ostream &
-FormationBPN::print( std::ostream & os ) const
+FormationBPN::printConf( std::ostream & os ) const
 {
-    printName( os );
-
     for ( int i = 0; i < 11; ++i )
     {
         const int unum = i + 1;
-        os << "player " << unum << " " << M_synmetry_number[i] << " ";
-        if ( M_synmetry_number[i] > 0 )
+
+        os << "player " << unum << ' ' << M_symmetry_number[i] << '\n';
+
+        if ( M_symmetry_number[i] > 0 )
         {
-            os << '\n';
             continue;
         }
-        os << '\n';
 
         std::map< int, boost::shared_ptr< FormationBPN::Param > >::const_iterator
             it = M_param_map.find( unum );
         if ( it == M_param_map.end() )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
                       << " *** ERROR *** Invalid player Id at number "
                       << i + 1
                       << ".  No formation param!"
@@ -873,4 +828,22 @@ FormationBPN::print( std::ostream & os ) const
     return os;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Formation::Ptr
+create()
+{
+    Formation::Ptr ptr( new FormationBPN() );
+    return ptr;
+}
+
+rcss::RegHolder f = Formation::creators().autoReg( &create,
+                                                   FormationBPN::NAME );
+
+}
+
 }
