@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_dt.cpp
-	\brief Delaunay Triangulation formation data classes Source File.
+  \file formation_dt.cpp
+  \brief Delaunay Triangulation formation data classes Source File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -39,41 +39,25 @@
 #include <rcsc/geom/line_2d.h>
 #include <rcsc/math_util.h>
 
+#include <sstream>
+
 namespace rcsc {
 
-/*-------------------------------------------------------------------*/
-/*!
+using namespace formation;
 
- */
-FormationDT::Param::Param( const Snapshot & data )
-    : ball_( data.ball_ )
-    , players_( data.players_ )
+const std::string FormationDT::NAME( "DelaunayTriangulation" );
+
+namespace {
+
+inline
+double
+round_coord( const double & val )
 {
-
+    return rint( val / SampleData::PRECISION ) * SampleData::PRECISION;
 }
 
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-Vector2D
-FormationDT::Param::getPosition( const int unum ) const
-{
-    if ( unum < 1 || 11 < unum )
-    {
-        std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** invalid unum " << unum
-                  << std::endl;
-        return Vector2D( 0.0, 0.0 );
-    }
-
-    return players_[unum - 1];
 }
 
-/*-------------------------------------------------------------------*/
-/*-------------------------------------------------------------------*/
-/*-------------------------------------------------------------------*/
-
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -81,6 +65,8 @@ FormationDT::Param::getPosition( const int unum ) const
 FormationDT::FormationDT()
     : Formation()
 {
+    M_version = 2;
+
     for ( int i = 0; i < 11; ++i )
     {
         M_role_name[i] = "Dummy";
@@ -93,8 +79,8 @@ FormationDT::FormationDT()
 /*!
 
  */
-Formation::Snapshot
-FormationDT::createDefaultParam()
+void
+FormationDT::createDefaultData()
 {
 #if 1
     // 4-3-3
@@ -112,30 +98,30 @@ FormationDT::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    setSynmetryType( 3, 2 );
+    setSymmetryType( 3, 2, "CenterBack" );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    setSynmetryType( 5, 4 );
+    setSymmetryType( 5, 4, "SideBack" );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    setSynmetryType( 8, 7 );
+    setSymmetryType( 8, 7, "OffensiveHalf" );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    setSynmetryType( 10, 9 );
+    setSymmetryType( 10, 9, "SideForward" );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    data.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 0.0 ) );
 
 #elif 0
     // 1-3-4-2
@@ -154,29 +140,29 @@ FormationDT::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
+    setSymmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::CENTER );
     createNewRole( 6, "OffensiveHalf", Formation::SIDE );
-    setSynmetryType( 7, 6 );
+    setSymmetryType( 7, 6 );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setSynmetryType( 9, 8 );
+    setSymmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
+    setSymmetryType( 11, 10 );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -9.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, -16.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, 16.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -25.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 25.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -10.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 10.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -9.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -5.0, -16.0 ) );
+    data.players_.push_back( Vector2D( -5.0, 16.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -25.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 25.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -10.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 10.0 ) );
 #else
     // 3-5-2
 
@@ -194,31 +180,32 @@ FormationDT::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
+    setSymmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setSynmetryType( 6, 5 );
+    setSymmetryType( 6, 5 );
     createNewRole( 7, "OffensiveHalf", Formation::CENTER );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setSynmetryType( 9, 8 );
+    setSymmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
+    setSymmetryType( 11, 10 );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, -16.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, 16.0 ) );
-    snap.players_.push_back( Vector2D( -9.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -25.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 25.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -10.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 10.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -5.0, -16.0 ) );
+    data.players_.push_back( Vector2D( -5.0, 16.0 ) );
+    data.players_.push_back( Vector2D( -9.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -25.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 25.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -10.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 10.0 ) );
 #endif
-    return snap;
+
+    M_samples->addData( *this, data, false );
 }
 
 /*-------------------------------------------------------------------*/
@@ -284,7 +271,7 @@ FormationDT::createNewRole( const int unum,
     case Formation::SIDE:
         setSideType( unum );
         break;
-    case Formation::SYNMETRY:
+    case Formation::SYMMETRY:
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** Invalid side type "
                   << std::endl;
@@ -346,6 +333,7 @@ FormationDT::interpolate( const int unum,
                           const Vector2D & focus_point,
                           const DelaunayTriangulation::Triangle * tri ) const
 {
+
     if ( ! tri )
     {
         const DelaunayTriangulation::Vertex * v
@@ -362,7 +350,7 @@ FormationDT::interpolate( const int unum,
         try
         {
             //std::cerr << "found nearest vertex id= " << v->id() << std::endl;
-            return M_param.at( v->id() ).getPosition( unum );
+            return M_sample_vector.at( v->id() ).getPosition( unum );
         }
         catch ( std::exception & e )
         {
@@ -375,9 +363,9 @@ FormationDT::interpolate( const int unum,
 
     try
     {
-        Vector2D result_0 = M_param.at( tri->vertex( 0 )->id() ).getPosition( unum );
-        Vector2D result_1 = M_param.at( tri->vertex( 1 )->id() ).getPosition( unum );
-        Vector2D result_2 = M_param.at( tri->vertex( 2 )->id() ).getPosition( unum );
+        Vector2D result_0 = M_sample_vector.at( tri->vertex( 0 )->id() ).getPosition( unum );
+        Vector2D result_1 = M_sample_vector.at( tri->vertex( 1 )->id() ).getPosition( unum );
+        Vector2D result_2 = M_sample_vector.at( tri->vertex( 2 )->id() ).getPosition( unum );
 
         Line2D line_0( tri->vertex( 0 )->pos(),
                        focus_point );
@@ -442,22 +430,25 @@ FormationDT::interpolate( const int unum,
 
  */
 void
-FormationDT::train( const std::list< Snapshot > & train_data )
+FormationDT::train()
 {
+    if ( ! M_samples )
+    {
+        return;
+    }
+
     Rect2D pitch( Vector2D( -60.0, -45.0 ),
                   Size2D( 120.0, 90.0 ) );
     M_triangulation.init( pitch );
-    M_param.clear();
+    M_sample_vector.clear();
 
-    const std::list< Formation::Snapshot >::const_iterator end
-        = train_data.end();
-    for ( std::list< Formation::Snapshot >::const_iterator it
-              = train_data.begin();
+    const SampleDataSet::DataCont::const_iterator end = M_samples->dataCont().end();
+    for ( SampleDataSet::DataCont::const_iterator it = M_samples->dataCont().begin();
           it != end;
           ++it )
     {
         M_triangulation.addVertex( it->ball_ );
-        M_param.push_back( Param( *it ) );
+        M_sample_vector.push_back( *it );
     }
 
     M_triangulation.compute();
@@ -468,19 +459,49 @@ FormationDT::train( const std::list< Snapshot > & train_data )
 
  */
 bool
-FormationDT::read( std::istream & is )
+FormationDT::readConf( std::istream & is )
 {
-    M_param.clear();
-
-    int n_line = 0;
-
-    // read formation type name
-    n_line += readName( is );
-    if ( n_line < 0 )
+    bool result = false;
+    if ( version() >= 2 )
     {
-        return false;
+        result = readV2( is );
+    }
+    else
+    {
+        result = readV1( is );
     }
 
+    if ( version() == 0 )
+    {
+        M_version = 2;
+    }
+
+    ////////////////////////////////////////////////////////
+    if ( result )
+    {
+        train();
+    }
+
+    return result;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationDT::readSamples( std::istream & )
+{
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationDT::readV1( std::istream & is )
+{
     // read role assignment
     if ( ! readRoles( is ) )
     {
@@ -492,14 +513,14 @@ FormationDT::read( std::istream & is )
     for ( int unum = 1; unum <= 11; ++unum )
     {
         std::cerr << "Role=" << getRoleName( unum )
-                  << " Synmetry=" << getSynmetryNumber( unum )
+                  << " Symmetry=" << getSymmetryNumber( unum )
                   << std::endl;
     }
 #endif
 
     //---------------------------------------------------
-    // read sample data
-    if ( ! readSamples( is ) )
+    // read vertices
+    if ( ! readVertices( is ) )
     {
         return false;
     }
@@ -510,22 +531,6 @@ FormationDT::read( std::istream & is )
                   << std::endl;
     }
 
-    ////////////////////////////////////////////////////////
-    {
-        Rect2D pitch( Vector2D( -60.0, -45.0 ),
-                      Size2D( 120.0, 90.0 ) );
-        M_triangulation.init( pitch );
-
-        for ( std::vector< Param >::const_iterator it = M_param.begin();
-              it != M_param.end();
-              ++it )
-        {
-            M_triangulation.addVertex( it->ball_ );
-        }
-
-        M_triangulation.compute();
-    }
-
     return true;
 }
 
@@ -540,7 +545,7 @@ FormationDT::readRoles( std::istream & is )
 
     if ( ! std::getline( is, line_buf ) )
     {
-        std::cerr << __FILE__ << ":" << __LINE__
+        std::cerr << __FILE__ << ':' << __LINE__ << ':'
                   << " *** ERROR *** readRoles(). Failed getline "
                   << std::endl;
         return false;
@@ -551,13 +556,13 @@ FormationDT::readRoles( std::istream & is )
     for ( int unum = 1; unum <= 11; ++unum )
     {
         char role_name[128];
-        int synmetry_number;
+        int symmetry_number;
         int n_read = 0;
 
         if ( std::sscanf( msg, " %s %d %n ",
-                          role_name, &synmetry_number, &n_read ) != 2 )
+                          role_name, &symmetry_number, &n_read ) != 2 )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
                       << " *** ERROR *** readRoles(). Failed to read player "
                       << unum
                       << std::endl;
@@ -565,11 +570,11 @@ FormationDT::readRoles( std::istream & is )
         }
         msg += n_read;
 
-        const Formation::SideType type = ( synmetry_number == 0
+        const Formation::SideType type = ( symmetry_number == 0
                                            ? Formation::CENTER
-                                           : synmetry_number < 0
+                                           : symmetry_number < 0
                                            ? Formation::SIDE
-                                           : Formation::SYNMETRY );
+                                           : Formation::SYMMETRY );
         if ( type == Formation::CENTER )
         {
             createNewRole( unum, role_name, type );
@@ -580,7 +585,7 @@ FormationDT::readRoles( std::istream & is )
         }
         else
         {
-            setSynmetryType( unum, synmetry_number );
+            setSymmetryType( unum, symmetry_number, role_name );
         }
     }
 
@@ -592,19 +597,20 @@ FormationDT::readRoles( std::istream & is )
 
  */
 bool
-FormationDT::readSamples( std::istream & is )
+FormationDT::readVertices( std::istream & is )
 {
+    M_samples = SampleDataSet::Ptr( new SampleDataSet() );
+
     std::string line_buf;
 
     while ( std::getline( is, line_buf ) )
     {
         if ( line_buf == "End" )
         {
-            break;
+            return true;
         }
 
-        M_param.push_back( Param() );
-
+        SampleData new_sample;
 
         const char * msg = line_buf.c_str();
 
@@ -615,34 +621,222 @@ FormationDT::readSamples( std::istream & is )
         if ( std::sscanf( msg, " %lf %lf %n ",
                           &read_x, &read_y, &n_read ) != 2 )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
                       << " *** ERROR *** Invalid ball data ["
                       << line_buf << "]"
                       << std::endl;
-            M_param.pop_back();
             return false;
         }
         msg += n_read;
 
-        M_param.back().ball_.assign( read_x, read_y );
+        new_sample.ball_.assign( read_x, read_y );
 
         for ( int unum = 1; unum <= 11; ++unum )
         {
             if ( std::sscanf( msg, " %lf %lf %n ",
                               &read_x, &read_y, &n_read ) != 2 )
             {
-                std::cerr << __FILE__ << ":" << __LINE__
-                          << " *** ERROR *** Invalid player data. unum = "
+                std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                          << " *** ERROR *** Illegal player data. unum = "
                           << unum
                           << " [" << line_buf << "]"
                           << std::endl;
-                M_param.pop_back();
                 return false;
             }
             msg += n_read;
 
-            M_param.back().players_[unum - 1].assign( read_x, read_y );
+            new_sample.players_.push_back( Vector2D( read_x, read_y ) );
         }
+
+        M_samples->addData( *this, new_sample, false );
+    }
+
+    return false;
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationDT::readV2( std::istream & is )
+{
+    if ( ! readRolesV2( is ) )
+    {
+        return false;
+    }
+
+    if ( ! readVerticesV2( is ) )
+    {
+        return false;
+    }
+
+    //
+    // read End tag
+    //
+
+    std::string line_buf;
+    while ( std::getline( is, line_buf ) )
+    {
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
+        {
+            continue;
+        }
+
+        if ( line_buf != "End" )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readV2(). No end tag "
+                      << std::endl;
+            return false;
+        }
+
+        break;
+    }
+
+    if ( is.eof() )
+    {
+        std::cerr << "Input stream reaches EOF"
+                  << std::endl;
+    }
+
+    return true;
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationDT::readRolesV2( std::istream & is )
+{
+    std::string line_buf;
+
+    //
+    // read Begin tag
+    //
+
+    while ( std::getline( is, line_buf ) )
+    {
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
+        {
+            continue;
+        }
+
+        if ( line_buf != "Begin Roles" )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRolesV2(). Illegal header ["
+                      << line_buf << ']'
+                      << std::endl;
+            return false;
+        }
+
+        break;
+    }
+
+    //
+    // read role data
+    //
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        while ( std::getline( is, line_buf ) )
+        {
+            if ( line_buf.empty()
+                 || line_buf[0] == '#'
+                 || ! line_buf.compare( 0, 2, "//" ) )
+            {
+                continue;
+            }
+            break;
+        }
+
+        int read_unum = 0;
+        char role_name[128];
+        int symmetry_number = 0;
+
+        if ( std::sscanf( line_buf.c_str(),
+                          " %d %127s %d ",
+                          &read_unum, role_name, &symmetry_number ) != 3
+             || read_unum != unum )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRolesV2(). Illegal role data. num="
+                      << unum
+                      << " [" << line_buf << "]"
+                      << std::endl;
+            return false;
+        }
+
+        //
+        // create role or set symmetry.
+        //
+        const Formation::SideType type = ( symmetry_number == 0
+                                           ? Formation::CENTER
+                                           : symmetry_number < 0
+                                           ? Formation::SIDE
+                                           : Formation::SYMMETRY );
+        if ( type == Formation::CENTER )
+        {
+            createNewRole( unum, role_name, type );
+        }
+        else if ( type == Formation::SIDE )
+        {
+            createNewRole( unum, role_name, type );
+        }
+        else
+        {
+            setSymmetryType( unum, symmetry_number, role_name );
+        }
+    }
+
+    //
+    // read End tag
+    //
+
+    while ( std::getline( is, line_buf ) )
+    {
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
+        {
+            continue;
+        }
+
+        if ( line_buf != "End Roles" )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRolesV2(). Failed getline "
+                      << std::endl;
+            return false;
+        }
+
+        break;
+    }
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationDT::readVerticesV2( std::istream & is )
+{
+    M_samples = SampleDataSet::Ptr( new SampleDataSet() );
+
+    if ( ! M_samples->read( is ) )
+    {
+        M_samples.reset();
+        return false;
     }
 
     return true;
@@ -653,32 +847,49 @@ FormationDT::readSamples( std::istream & is )
 
  */
 std::ostream &
-FormationDT::print( std::ostream & os ) const
+FormationDT::printConf( std::ostream & os ) const
 {
-    printName( os );
+    return printV2( os );
+}
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationDT::printSamples( std::ostream & os ) const
+{
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationDT::printV1( std::ostream & os ) const
+{
     // put role assignment
     for ( int unum = 1; unum <= 11; ++unum )
     {
         os << M_role_name[unum - 1] << ' '
-           << M_synmetry_number[unum - 1] << ' ';
+           << M_symmetry_number[unum - 1] << ' ';
     }
     os << '\n';
 
-    for ( std::vector< Param >::const_iterator it = M_param.begin();
-          it != M_param.end();
+    for ( std::vector< SampleData >::const_iterator it = M_sample_vector.begin();
+          it != M_sample_vector.end();
           ++it )
     {
-        os << it->ball_.x << ' '
-           << it->ball_.y << ' ';
+        os << round_coord( it->ball_.x ) << ' '
+           << round_coord( it->ball_.y ) << ' ';
 
-        for ( std::vector< Vector2D >::const_iterator p
-                  = it->players_.begin();
+        for ( SampleData::PlayerCont::const_iterator p = it->players_.begin();
               p != it->players_.end();
               ++p )
         {
-            os << p->x << ' '
-               << p->y << ' ';
+            os << round_coord( p->x ) << ' '
+               << round_coord( p->y ) << ' ';
         }
         os << '\n';
     }
@@ -687,4 +898,68 @@ FormationDT::print( std::ostream & os ) const
     return os;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationDT::printV2( std::ostream & os ) const
+{
+    printRolesV2( os );
+    printDataV2( os );
+
+    os << "End" << std::endl;
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationDT::printRolesV2( std::ostream & os ) const
+{
+    os << "Begin Roles\n";
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        os << unum << ' '
+           << M_role_name[unum - 1] << ' '
+           << M_symmetry_number[unum - 1] << '\n';
+    }
+
+    os << "End Roles\n";
+
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationDT::printDataV2( std::ostream & os ) const
+{
+    M_samples->print( os );
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Formation::Ptr
+create()
+{
+    Formation::Ptr ptr( new FormationDT() );
+    return ptr;
+}
+
+rcss::RegHolder f = Formation::creators().autoReg( &create,
+                                                   FormationDT::NAME );
+
+}
+
 }
