@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_knn.cpp
-	\brief k-nearest neighbor formation class Source File.
+  \file formation_knn.cpp
+  \brief k-nearest neighbor formation class Source File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,75 +41,9 @@
 
 namespace rcsc {
 
-/*-------------------------------------------------------------------*/
-/*!
+using namespace formation;
 
- */
-FormationKNN::Data::Data()
-    : ball_( 0.0, 0.0 )
-    , players_( 11 )
-{
-
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-FormationKNN::Data::Data( const Snapshot & snapshot )
-    : ball_( snapshot.ball_ )
-    , players_( snapshot.players_ )
-{
-
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-FormationKNN::Data::Data( const Vector2D & ball,
-                          const std::vector< Vector2D > & players )
-    : ball_( ball )
-    , players_( players )
-{
-
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-const
-FormationKNN::Data &
-FormationKNN::Data::assign( const Vector2D & ball,
-                            const std::vector< Vector2D > & players )
-{
-    ball_ = ball;
-    players_ = players;
-    return *this;
-}
-
-/*-------------------------------------------------------------------*/
-/*!
-
- */
-Vector2D
-FormationKNN::Data::getPosition( const int unum ) const
-{
-    if ( unum < 1 || 11 < unum )
-    {
-        std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** invalid unum " << unum
-                  << std::endl;
-        return Vector2D( 0.0, 0.0 );
-    }
-
-    return players_[unum - 1];
-}
-
-/*-------------------------------------------------------------------*/
-/*-------------------------------------------------------------------*/
-/*-------------------------------------------------------------------*/
+const std::string FormationKNN::NAME( "k-NN" );
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -123,16 +57,16 @@ FormationKNN::FormationKNN()
     {
         M_role_name[i] = "Dummy";
     }
+
+    M_version = 2;
 }
 
-
-
 /*-------------------------------------------------------------------*/
 /*!
 
  */
-Formation::Snapshot
-FormationKNN::createDefaultParam()
+void
+FormationKNN::createDefaultData()
 {
 #if 1
     // 4-3-3
@@ -150,30 +84,30 @@ FormationKNN::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    setSynmetryType( 3, 2 );
+    setSymmetryType( 3, 2, "CenterBack" );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    setSynmetryType( 5, 4 );
+    setSymmetryType( 5, 4, "SideBack" );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    setSynmetryType( 8, 7 );
+    setSymmetryType( 8, 7, "OffensiveHalf" );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    setSynmetryType( 10, 9 );
+    setSymmetryType( 10, 9, "SideForward" );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    data.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 0.0 ) );
 
 #elif 0
     // 1-3-4-2
@@ -192,29 +126,29 @@ FormationKNN::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
+    setSymmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::CENTER );
     createNewRole( 6, "OffensiveHalf", Formation::SIDE );
-    setSynmetryType( 7, 6 );
+    setSymmetryType( 7, 6 );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setSynmetryType( 9, 8 );
+    setSymmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
+    setSymmetryType( 11, 10 );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -9.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, -16.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, 16.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -25.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 25.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -10.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 10.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -9.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -5.0, -16.0 ) );
+    data.players_.push_back( Vector2D( -5.0, 16.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -25.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 25.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -10.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 10.0 ) );
 #else
     // 3-5-2
 
@@ -232,31 +166,32 @@ FormationKNN::createDefaultParam()
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "Sweeper", Formation::CENTER );
     createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
+    setSymmetryType( 4, 3 );
     createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setSynmetryType( 6, 5 );
+    setSymmetryType( 6, 5 );
     createNewRole( 7, "OffensiveHalf", Formation::CENTER );
     createNewRole( 8, "SideHalf", Formation::SIDE );
-    setSynmetryType( 9, 8 );
+    setSymmetryType( 9, 8 );
     createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
+    setSymmetryType( 11, 10 );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, -16.0 ) );
-    snap.players_.push_back( Vector2D( -5.0, 16.0 ) );
-    snap.players_.push_back( Vector2D( -9.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -25.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 25.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -10.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 10.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -15.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -5.0, -16.0 ) );
+    data.players_.push_back( Vector2D( -5.0, 16.0 ) );
+    data.players_.push_back( Vector2D( -9.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -25.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 25.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -10.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 10.0 ) );
 #endif
-    return snap;
+
+    M_samples->addData( *this, data, false );
 }
 
 /*-------------------------------------------------------------------*/
@@ -322,7 +257,7 @@ FormationKNN::createNewRole( const int unum,
     case Formation::SIDE:
         setSideType( unum );
         break;
-    case Formation::SYNMETRY:
+    case Formation::SYMMETRY:
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** Unsupported side type "
                   << std::endl;
@@ -358,8 +293,8 @@ public:
       \param rhs right hand side object
       \return true if lhs data point is closer than rhs data point
      */
-    bool operator()( const FormationKNN::Data * lhs,
-                     const FormationKNN::Data * rhs )
+    bool operator()( const SampleData * lhs,
+                     const SampleData * rhs )
       {
           return lhs->ball_.dist2( M_point ) < rhs->ball_.dist2( M_point );
       }
@@ -374,6 +309,8 @@ Vector2D
 FormationKNN::getPosition( const int unum,
                            const Vector2D & focus_point ) const
 {
+    static std::vector< const SampleData * > ptr_vector;
+
     if ( unum < 1 || 11 < unum )
     {
         std::cerr << __FILE__ << ":" << __LINE__
@@ -382,16 +319,26 @@ FormationKNN::getPosition( const int unum,
         return Vector2D::INVALIDATED;
     }
 
-    if ( M_data_ptr.empty() )
+    if ( M_samples->dataCont().empty() )
     {
         return Vector2D( 0.0, 0.0 );
     }
 
-    const size_t size = std::min( M_data_ptr.size(), M_k );
+    ptr_vector.reserve( M_samples->dataCont().size() );
 
-    std::partial_sort( M_data_ptr.begin(),
-                       M_data_ptr.begin() + size,
-                       M_data_ptr.end(),
+    const SampleDataSet::DataCont::const_iterator d_end = M_samples->dataCont().end();
+    for ( SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
+          d != d_end;
+          ++d )
+    {
+        ptr_vector.push_back( &(*d) );
+    }
+
+    const size_t size = std::min( ptr_vector.size(), M_k );
+
+    std::partial_sort( ptr_vector.begin(),
+                       ptr_vector.begin() + size,
+                       ptr_vector.end(),
                        DataCmp( focus_point ) );
 
     std::vector< double > inv_dist2( size, 0.0 );
@@ -399,10 +346,10 @@ FormationKNN::getPosition( const int unum,
 
     for ( size_t i = 0; i < size; ++i )
     {
-        double d2 = M_data_ptr[i]->ball_.dist2( focus_point );
+        double d2 = ptr_vector[i]->ball_.dist2( focus_point );
         if ( d2 < 1.0e-10 )
         {
-            return M_data_ptr[i]->getPosition( unum );
+            return ptr_vector[i]->getPosition( unum );
         }
         inv_dist2[i] = 1.0 / d2;
         sum_inv_dist2 += inv_dist2[i];
@@ -412,18 +359,21 @@ FormationKNN::getPosition( const int unum,
 
     for ( size_t i = 0; i < size; ++i )
     {
-        pos += M_data_ptr[i]->getPosition( unum ) * inv_dist2[i];
+        pos += ptr_vector[i]->getPosition( unum ) * inv_dist2[i];
     }
 
     pos /= sum_inv_dist2;
+
+    ptr_vector.clear();
+
 #if 0
     if ( unum == 11 )
     {
         std::cerr << "sum_inv_dist2=" << sum_inv_dist2 << '\n';
         for ( size_t i = 0; i < size; ++i )
         {
-            std::cerr << "  ball=" << M_data_ptr[i]->ball_
-                      << " pos=" << M_data_ptr[i]->getPosition( unum )
+            std::cerr << "  ball=" << ptr_vector[i]->ball_
+                      << " pos=" << ptr_vector[i]->getPosition( unum )
                       << " inv_dist2=" << inv_dist2[i]
                       << " rate=" << ( inv_dist2[i] / sum_inv_dist2 )
                       << '\n';
@@ -431,6 +381,7 @@ FormationKNN::getPosition( const int unum,
         std::cerr << "  pos = " << pos << std::endl;
     }
 #endif
+
     return pos;
 }
 
@@ -442,22 +393,44 @@ void
 FormationKNN::getPositions( const Vector2D & focus_point,
                             std::vector< Vector2D > & positions ) const
 {
+    static std::vector< const SampleData * > ptr_vector;
+
     positions.clear();
 
-    const size_t size = std::min( M_data_ptr.size(), M_k );
+    ptr_vector.reserve( M_samples->dataCont().size() );
 
-    std::partial_sort( M_data_ptr.begin(),
-                       M_data_ptr.begin() + size,
-                       M_data_ptr.end(),
+    const SampleDataSet::DataCont::const_iterator d_end = M_samples->dataCont().end();
+    for ( SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
+          d != d_end;
+          ++d )
+    {
+        ptr_vector.push_back( &(*d) );
+    }
+
+    const size_t size = std::min( ptr_vector.size(), M_k );
+
+    std::partial_sort( ptr_vector.begin(),
+                       ptr_vector.begin() + size,
+                       ptr_vector.end(),
                        DataCmp( focus_point ) );
 
-    std::vector< double > dist( size, 0.0 );
-    double sum_dist = 0.0;
+    std::vector< double > inv_dist2( size, 0.0 );
+    double sum_inv_dist2 = 0.0;
 
     for ( size_t i = 0; i < size; ++i )
     {
-        dist[i] = M_data_ptr[i]->ball_.dist( focus_point );
-        sum_dist += dist[i];
+        double d2 = ptr_vector[i]->ball_.dist2( focus_point );
+        if ( d2 < 1.0e-10 )
+        {
+            for ( int unum = 1; unum <= 11; ++unum )
+            {
+                positions.push_back( ptr_vector[i]->getPosition( unum ) );
+            }
+            return;
+        }
+
+        inv_dist2[i] = 1.0 / d2;
+        sum_inv_dist2 += inv_dist2[i];
     }
 
     for ( int unum = 1; unum <= 11; ++unum )
@@ -466,11 +439,14 @@ FormationKNN::getPositions( const Vector2D & focus_point,
 
         for ( size_t i = 0; i < size; ++i )
         {
-            pos += M_data_ptr[i]->getPosition( unum ) * ( ( sum_dist - dist[i] ) / sum_dist );
+            pos += ptr_vector[i]->getPosition( unum ) * inv_dist2[i];
         }
 
+        pos /= sum_inv_dist2;
         positions.push_back( pos );
     }
+
+    ptr_vector.clear();
 }
 
 /*-------------------------------------------------------------------*/
@@ -478,29 +454,9 @@ FormationKNN::getPositions( const Vector2D & focus_point,
 
  */
 void
-FormationKNN::train( const std::list< Snapshot > & train_data )
+FormationKNN::train()
 {
-    M_data.clear();
-    M_data_ptr.clear();
 
-    M_data.reserve( train_data.size() );
-
-    for ( std::list< Formation::Snapshot >::const_iterator it
-              = train_data.begin();
-          it != train_data.end();
-          ++it )
-    {
-        M_data.push_back( Data( *it ) );
-    }
-
-    M_data.reserve( M_data.size() );
-
-    for ( std::vector< Data >::iterator it = M_data.begin();
-          it != M_data.end();
-          ++it )
-    {
-        M_data_ptr.push_back( &(*it) );
-    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -508,21 +464,8 @@ FormationKNN::train( const std::list< Snapshot > & train_data )
 
  */
 bool
-FormationKNN::read( std::istream & is )
+FormationKNN::readConf( std::istream & is )
 {
-    M_data.clear();
-
-    int n_line = 0;
-
-    // read formation type name
-    n_line += readName( is );
-    if ( n_line < 0 )
-    {
-        return false;
-    }
-
-    std::string line_buf;
-
     // read role assignment
     if ( ! readRoles( is ) )
     {
@@ -530,8 +473,8 @@ FormationKNN::read( std::istream & is )
     }
 
     //---------------------------------------------------
-    // read sample data
-    if ( ! readSamples( is ) )
+    // read kernel point data
+    if ( ! readKernelPoints( is ) )
     {
         return false;
     }
@@ -539,6 +482,16 @@ FormationKNN::read( std::istream & is )
     return true;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+FormationKNN::readSamples( std::istream & )
+{
+    return true;
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -548,38 +501,73 @@ FormationKNN::readRoles( std::istream & is )
 {
     std::string line_buf;
 
-    if ( ! std::getline( is, line_buf ) )
+    //
+    // read Begin tag
+    //
+
+    while ( std::getline( is, line_buf ) )
     {
-        std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** readRoles(). Failed getline "
-                  << std::endl;
-        return false;
-    }
-
-    const char * msg = line_buf.c_str();
-
-    for ( int unum = 1; unum <= 11; ++unum )
-    {
-        char role_name[128];
-        int synmetry_number;
-        int n_read = 0;
-
-        if ( std::sscanf( msg, " %s %d %n ",
-                          role_name, &synmetry_number, &n_read ) != 2 )
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
         {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** readRoles(). Failed to read player "
-                      << unum
+            continue;
+        }
+
+        if ( line_buf != "Begin Roles" )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRoles(). Illegal header ["
+                      << line_buf << ']'
                       << std::endl;
             return false;
         }
-        msg += n_read;
 
-        const Formation::SideType type = ( synmetry_number == 0
+        break;
+    }
+
+    //
+    // read role data
+    //
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        while ( std::getline( is, line_buf ) )
+        {
+            if ( line_buf.empty()
+                 || line_buf[0] == '#'
+                 || ! line_buf.compare( 0, 2, "//" ) )
+            {
+                continue;
+            }
+            break;
+        }
+
+        int read_unum = 0;
+        char role_name[128];
+        int symmetry_number = 0;
+
+        if ( std::sscanf( line_buf.c_str(),
+                          " %d %127s %d ",
+                          &read_unum, role_name, &symmetry_number ) != 3
+             || read_unum != unum )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRoles(). Illegal role data. num="
+                      << unum
+                      << " [" << line_buf << "]"
+                      << std::endl;
+            return false;
+        }
+
+        //
+        // create role or set symmetry.
+        //
+        const Formation::SideType type = ( symmetry_number == 0
                                            ? Formation::CENTER
-                                           : synmetry_number < 0
+                                           : symmetry_number < 0
                                            ? Formation::SIDE
-                                           : Formation::SYNMETRY );
+                                           : Formation::SYMMETRY );
         if ( type == Formation::CENTER )
         {
             createNewRole( unum, role_name, type );
@@ -590,10 +578,34 @@ FormationKNN::readRoles( std::istream & is )
         }
         else
         {
-            setSynmetryType( unum, synmetry_number );
+            setSymmetryType( unum, symmetry_number, role_name );
         }
     }
 
+    //
+    // read End tag
+    //
+
+    while ( std::getline( is, line_buf ) )
+    {
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
+        {
+            continue;
+        }
+
+        if ( line_buf != "End Roles" )
+        {
+            std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                      << " *** ERROR *** readRoles(). Failed getline "
+                      << std::endl;
+            return false;
+        }
+
+        break;
+    }
+
     return true;
 }
 
@@ -602,66 +614,14 @@ FormationKNN::readRoles( std::istream & is )
 
  */
 bool
-FormationKNN::readSamples( std::istream & is )
+FormationKNN::readKernelPoints( std::istream & is )
 {
-    std::string line_buf;
+    M_samples = SampleDataSet::Ptr( new formation::SampleDataSet() );
 
-    while ( std::getline( is, line_buf ) )
+    if ( ! M_samples->read( is ) )
     {
-        if ( line_buf == "End" )
-        {
-            break;
-        }
-
-        M_data.push_back( Data() );
-
-        const char * msg = line_buf.c_str();
-
-        double read_x, read_y;
-        int n_read = 0;
-
-        // read ball pos
-        if ( std::sscanf( msg, " %lf %lf %n ",
-                          &read_x, &read_y, &n_read ) != 2 )
-        {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** Invalid ball data ["
-                      << line_buf << "]"
-                      << std::endl;
-            M_data.pop_back();
-            return false;
-        }
-        msg += n_read;
-
-        M_data.back().ball_.assign( read_x, read_y );
-
-        for ( int unum = 1; unum <= 11; ++unum )
-        {
-            if ( std::sscanf( msg, " %lf %lf %n ",
-                              &read_x, &read_y, &n_read ) != 2 )
-            {
-                std::cerr << __FILE__ << ":" << __LINE__
-                          << " *** ERROR *** Invalid player data. unum = "
-                          << unum
-                          << " [" << line_buf << "]"
-                          << std::endl;
-                M_data.pop_back();
-                return false;
-            }
-            msg += n_read;
-
-            M_data.back().players_[unum - 1].assign( read_x, read_y );
-        }
-    }
-
-
-    M_data_ptr.reserve( M_data.size() );
-
-    for ( std::vector< Data >::iterator it = M_data.begin();
-          it != M_data.end();
-          ++it )
-    {
-        M_data_ptr.push_back( &(*it) );
+        M_samples.reset();
+        return false;
     }
 
     return true;
@@ -672,38 +632,62 @@ FormationKNN::readSamples( std::istream & is )
 
  */
 std::ostream &
-FormationKNN::print( std::ostream & os ) const
+FormationKNN::printConf( std::ostream & os ) const
 {
-    printName( os );
-
-    // put role assignment
-    for ( int unum = 1; unum <= 11; ++unum )
-    {
-        os << M_role_name[unum - 1] << ' '
-           << M_synmetry_number[unum - 1] << ' ';
-    }
-    os << '\n';
-
-    for ( std::vector< Data >::const_iterator it = M_data.begin();
-          it != M_data.end();
-          ++it )
-    {
-        os << it->ball_.x << ' '
-           << it->ball_.y << ' ';
-
-        for ( std::vector< Vector2D >::const_iterator p
-                  = it->players_.begin();
-              p != it->players_.end();
-              ++p )
-        {
-            os << p->x << ' '
-               << p->y << ' ';
-        }
-        os << '\n';
-    }
+    printRoles( os );
+    M_samples->print( os );
 
     os << "End" << std::endl;
     return os;
 }
 
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationKNN::printSamples( std::ostream & os ) const
+{
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+FormationKNN::printRoles( std::ostream & os ) const
+{
+    os << "Begin Roles\n";
+
+    for ( int unum = 1; unum <= 11; ++unum )
+    {
+        os << unum << ' '
+           << M_role_name[unum - 1] << ' '
+           << M_symmetry_number[unum - 1] << '\n';
+    }
+
+    os << "End Roles\n";
+
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Formation::Ptr
+create()
+{
+    Formation::Ptr ptr( new FormationKNN() );
+    return ptr;
+}
+
+rcss::RegHolder f = Formation::creators().autoReg( &create,
+                                                   FormationKNN::NAME );
+
+}
+
 }
