@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_ngnet.cpp
-	\brief NGNet formation data classes Source File.
+  \file formation_ngnet.cpp
+  \brief NGNet formation data classes Source File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,6 +43,10 @@
 
 namespace rcsc {
 
+using namespace formation;
+
+const std::string FormationNGNet::NAME( "NGNet" );
+
 const double FormationNGNet::Param::PITCH_LENGTH = 105.0 + 10.0;
 const double FormationNGNet::Param::PITCH_WIDTH = 68.0 + 10.0;
 
@@ -50,7 +54,7 @@ const double FormationNGNet::Param::PITCH_WIDTH = 68.0 + 10.0;
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationNGNet::Param::Param()
     : M_net()
 {
@@ -60,10 +64,10 @@ FormationNGNet::Param::Param()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationNGNet::Param::getPosition( const Vector2D & ball_pos,
-                                    const Formation::SideType type ) const
+                                    const Formation::SideType ) const
 {
     NGNet::input_vector input;
 
@@ -123,7 +127,7 @@ FormationNGNet::Param::readRoleName( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationNGNet::Param::readParam( std::istream & is )
 {
@@ -155,7 +159,7 @@ FormationNGNet::Param::readParam( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationNGNet::Param::read( std::istream & is )
 {
@@ -180,7 +184,7 @@ FormationNGNet::Param::read( std::istream & is )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 FormationNGNet::Param::printRoleName( std::ostream & os ) const
 {
@@ -198,7 +202,7 @@ FormationNGNet::Param::printRoleName( std::ostream & os ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::ostream &
 FormationNGNet::Param::printParam( std::ostream & os ) const
 {
@@ -208,11 +212,6 @@ FormationNGNet::Param::printParam( std::ostream & os ) const
 
 /*-------------------------------------------------------------------*/
 /*!
-  Role <role name>
-  <bko.x> <bkoy>
-  <offense playon>
-  <defense playon>
-  ...
 
 */
 std::ostream &
@@ -231,7 +230,7 @@ FormationNGNet::Param::print( std::ostream & os ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 FormationNGNet::FormationNGNet()
     : Formation()
 {
@@ -241,11 +240,10 @@ FormationNGNet::FormationNGNet()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
-Formation::Snapshot
-FormationNGNet::createDefaultParam()
+ */
+void
+FormationNGNet::createDefaultData()
 {
-#if 1
     // 1: goalie
     // 2: left center back
     // 3(2): right center back
@@ -259,67 +257,39 @@ FormationNGNet::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    createNewRole( 3, "CenterBack", Formation::MIRROR );
-    setMirrorType( 3, 2 );
+    setSymmetryType( 3, 2, "CenterBack" );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    createNewRole( 5, "SideBack", Formation::MIRROR );
-    setMirrorType( 5, 4 );
+    setSymmetryType( 5, 4, "SideBack" );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    createNewRole( 8, "OffensiveHalf", Formation::MIRROR );
-    setMirrorType( 8, 7 );
+    setSymmetryType( 8, 7, "OffensiveHalf" );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    createNewRole( 10, "SideForward", Formation::MIRROR );
-    setMirrorType( 10, 9 );
+    setSymmetryType( 10, 9, "SideForward" );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    data.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 0.0 ) );
 
-    return snap;
-#else
-    // 1: goalie
-    // 2: center back
-    // 3: left side back
-    // 4(3): right side back
-    // 5: left defensive half
-    // 6(5): right defensive half
-    // 7: offensive half
-    // 8: left side half
-    // 9(8): right side half
-    // 10: left forward
-    // 11(10): right forward
-    createNewRole( 1, "Goalie", Formation::CENTER );
-    createNewRole( 2, "CenterBack", Formation::CENTER );
-    createNewRole( 3, "SideBack", Formation::SIDE );
-    setMirrorType( 4, 3 );
-    createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setMirrorType( 6, 5 );
-    createNewRole( 7, "SideHalf", Formation::SIDE );
-    setMirrorType( 8, 7 );
-    createNewRole( 9, "OffensiveHalf", Formation::CENTER );
-    createNewRole( 10, "Forward", Formation::SIDE );
-    setMirrorType( 11, 10 );
-#endif
+    M_samples->addData( *this, data, false );
 }
 
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationNGNet::setRoleName( const int unum,
                              const std::string & name )
@@ -341,7 +311,7 @@ FormationNGNet::setRoleName( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 std::string
 FormationNGNet::getRoleName( const int unum ) const
 {
@@ -361,7 +331,7 @@ FormationNGNet::getRoleName( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationNGNet::createNewRole( const int unum,
                                const std::string & role_name,
@@ -385,7 +355,7 @@ FormationNGNet::createNewRole( const int unum,
     }
     else
     {
-        // mirror
+        // symmetry
     }
 
     // erase old parameter, if exist
@@ -405,7 +375,7 @@ FormationNGNet::createNewRole( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 Vector2D
 FormationNGNet::getPosition( const int unum,
                              const Vector2D & ball_pos ) const
@@ -420,8 +390,8 @@ FormationNGNet::getPosition( const int unum,
         return Vector2D( 0.0, 0.0 );
     }
     Formation::SideType type = Formation::SIDE;
-    if ( M_mirror_number[unum - 1] > 0 )  type = Formation::MIRROR;
-    if ( M_mirror_number[unum - 1] == 0 ) type = Formation::CENTER;
+    if ( M_symmetry_number[unum - 1] > 0 )  type = Formation::SYMMETRY;
+    if ( M_symmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
 
     return ptr->getPosition( ball_pos, type );
 }
@@ -429,7 +399,7 @@ FormationNGNet::getPosition( const int unum,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 FormationNGNet::getPositions( const Vector2D & focus_point,
                               std::vector< Vector2D > & positions ) const
@@ -445,11 +415,12 @@ FormationNGNet::getPositions( const Vector2D & focus_point,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-FormationNGNet::train( const std::list< Snapshot > & train_data )
+FormationNGNet::train()
 {
-    if ( train_data.empty() )
+    if ( ! M_samples
+         || M_samples->dataCont().empty() )
     {
         return;
     }
@@ -475,36 +446,36 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
                   << net.units().size()
                   << std::endl;
 
-        if ( net.units().size() < train_data.size() )
+        if ( net.units().size() < M_samples->dataCont().size() )
         {
-            std::list< Snapshot >::const_iterator snap = train_data.begin();
+            SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
             int count = net.units().size();
             std::cerr << "FormationNGNet::train. need to add new center "
-                      << train_data.size() - net.units().size() << std::endl;
+                      << M_samples->dataCont().size() - net.units().size() << std::endl;
             while ( --count >= 0
-                    && snap != train_data.end() )
+                    && d != M_samples->dataCont().end() )
             {
                 std::cerr << "FormationNGNet::train. skip known data...." << std::endl;
-                ++snap;
+                ++d;
             }
 
-            while ( snap != train_data.end() )
+            while ( d != M_samples->dataCont().end() )
             {
                 std::cerr << "FormationNGNet::train. added new center "
-                          << snap->ball_
+                          << d->ball_
                           << std::endl;
                 NGNet::input_vector center;
                 //center[0] = bound( 0.0,
-                //                   snap->ball_.x / PITCH_LENGTH + 0.5,
+                //                   d->ball_.x / PITCH_LENGTH + 0.5,
                 //                   1.0 );
-                center[0] = snap->ball_.x;
+                center[0] = d->ball_.x;
                 //center[1] = bound( 0.0,
-                //                   snap->ball_.y / PITCH_WIDTH + 0.5,
+                //                   d->ball_.y / PITCH_WIDTH + 0.5,
                 //                   1.0 );
-                center[1] = snap->ball_.y;
+                center[1] = d->ball_.y;
                 net.addCenter( center );
 
-                ++snap;
+                ++d;
             }
         }
 
@@ -513,8 +484,7 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
         NGNet::input_vector input;
         NGNet::output_vector teacher;
 
-        const std::list< Formation::Snapshot >::const_iterator snap_end
-            = train_data.end();
+        const SampleDataSet::DataCont::const_iterator d_end = M_samples->dataCont().end();
         int loop = 0;
         double ave_err = 0.0;
         double max_err = 0.0;
@@ -523,11 +493,10 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
         {
             ave_err = 0.0;
             max_err = 0.0;
-            double snap_count = 1.0;
-            for ( std::list< Formation::Snapshot >::const_iterator snap
-                      = train_data.begin();
-                  snap != snap_end;
-                  ++snap, snap_count += 1.0 )
+            double data_count = 1.0;
+            for ( SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
+                  d != d_end;
+                  ++d, data_count += 1.0 )
             {
                 /*
                   input[0] = bound( 0.0,
@@ -543,10 +512,10 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
                   snap->players_[unum - 1].y / PITCH_WIDTH + 0.5,
                   1.0 );
                 */
-                input[0] = snap->ball_.x;
-                input[1] = snap->ball_.y;
-                teacher[0] = snap->players_[unum - 1].x;
-                teacher[1] = snap->players_[unum - 1].y;
+                input[0] = d->ball_.x;
+                input[1] = d->ball_.y;
+                teacher[0] = d->players_[unum - 1].x;
+                teacher[1] = d->players_[unum - 1].y;
 
                 if ( loop == 2 )
                 {
@@ -562,8 +531,8 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
                     max_err = err;
                 }
                 ave_err
-                    = ave_err * ( ( snap_count - 1.0 ) / snap_count )
-                    + err / snap_count;
+                    = ave_err * ( ( data_count - 1.0 ) / data_count )
+                    + err / data_count;
             }
 
             if ( max_err < 0.001 )
@@ -593,7 +562,7 @@ FormationNGNet::train( const std::list< Snapshot > & train_data )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 boost::shared_ptr< FormationNGNet::Param >
 FormationNGNet::getParam( const int unum )
 {
@@ -624,7 +593,7 @@ FormationNGNet::getParam( const int unum )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 boost::shared_ptr< const FormationNGNet::Param >
 FormationNGNet::param( const int unum ) const
 {
@@ -655,51 +624,22 @@ FormationNGNet::param( const int unum ) const
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
-FormationNGNet::read( std::istream & is )
+FormationNGNet::readConf( std::istream & is )
 {
-    int n_line = 0;
-
-    n_line += readName( is );
-    if ( n_line < 0 )
-    {
-        return false;
-    }
-
-    //---------------------------------------------------
-    // read each player's parameter set
     if ( ! readPlayers( is ) )
     {
         return false;
     }
 
-    //---------------------------------------------------
-    // check mirror number circuration reference
-    for ( int i = 0; i < 11; ++i )
-    {
-        int refered_unum = M_mirror_number[i];
-        if ( refered_unum <= 0 ) continue;
-        if ( M_mirror_number[refered_unum - 1] > 0 )
-        {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** failed to read formation."
-                      << " Bad mirroring. player "
-                      << i + 1
-                      << " mirro = " << refered_unum
-                      << " is already mirroring player"
-                      << std::endl;
-            return false;
-        }
-    }
-
     return true;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 bool
 FormationNGNet::readPlayers( std::istream & is )
 {
@@ -717,11 +657,11 @@ FormationNGNet::readPlayers( std::istream & is )
 
         // check id
         int unum = 0;
-        int mirror = 0;
+        int symmetry = 0;
         int n_read = 0;
         if ( std::sscanf( line_buf.c_str(),
                           " player %d %d %n ",
-                          &unum, &mirror,
+                          &unum, &symmetry,
                           &n_read ) != 2
              || n_read == 0 )
         {
@@ -741,34 +681,34 @@ FormationNGNet::readPlayers( std::istream & is )
                       << std::endl;
             return false;
         }
-        if ( mirror == unum )
+        if ( symmetry == unum )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid mirror number. at "
+                      << " Invalid symmetry number. at "
                       << i
                       << " mirroing player itself. unum = " << unum
-                      << "  mirror = " << mirror
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
-        if ( 11 < mirror )
+        if ( 11 < symmetry )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid mirror number. at "
+                      << " Invalid symmetry number. at "
                       << i
-                      << "  Mirror number is out of range. unum = " << unum
-                      << "  mirror = " << mirror
+                      << "  Symmetry number is out of range. unum = " << unum
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
 
-        M_mirror_number[i] = mirror;
+        M_symmetry_number[i] = symmetry;
 
-        // this player is mirror type
+        // this player is symmetry type
         /*
-          if ( mirror > 0 )
+          if ( symmetry > 0 )
           {
           ++player_read;
           continue;
@@ -800,36 +740,36 @@ FormationNGNet::readPlayers( std::istream & is )
         return false;
     }
 
+
+    if ( ! std::getline( is, line_buf )
+         || line_buf != "End" )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                  << " ***ERROR*** Illegal end tag."
+                  << std::endl;
+        return false;
+    }
+
     return true;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  condition <playmode> <>
-  player <unum> 0 <Role> <Param1> <Param2> ...
-  player <unum> <mirro> <Role>
-  ...
-  important <x> <y>
-  important <x> <y>
-  ...
+
 */
 std::ostream &
-FormationNGNet::print( std::ostream & os ) const
+FormationNGNet::printConf( std::ostream & os ) const
 {
-    printName( os );
-
     for ( int i = 0; i < 11; ++i )
     {
         const int unum = i + 1;
-        os << "player " << unum << " " << M_mirror_number[i] << " ";
+        os << "player " << unum << " " << M_symmetry_number[i] << '\n';
         /*
-          if ( M_mirror_number[i] > 0 )
+          if ( M_symmetry_number[i] > 0 )
           {
-          os << '\n';
-          continue;
+           continue;
           }
         */
-        os << '\n';
 
         std::map< int, boost::shared_ptr< FormationNGNet::Param > >::const_iterator
             it = M_param_map.find( unum );
@@ -851,4 +791,23 @@ FormationNGNet::print( std::ostream & os ) const
     return os;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Formation::Ptr
+create()
+{
+    Formation::Ptr ptr( new FormationNGNet() );
+    return ptr;
+}
+
+rcss::RegHolder f = Formation::creators().autoReg( &create,
+                                                   FormationNGNet::NAME );
+
+}
+
 }
