@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_rbf.cpp
-	\brief RBF formation data classes Source File.
+  \file formation_rbf.cpp
+  \brief RBF formation data classes Source File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -43,6 +43,10 @@
 
 namespace rcsc {
 
+using namespace formation;
+
+const std::string FormationRBF::NAME( "RBF" );
+
 const double FormationRBF::Param::PITCH_LENGTH = 105.0 + 10.0;
 const double FormationRBF::Param::PITCH_WIDTH = 68.0 + 10.0;
 
@@ -242,10 +246,9 @@ FormationRBF::FormationRBF()
 /*!
 
  */
-Formation::Snapshot
-FormationRBF::createDefaultParam()
+void
+FormationRBF::createDefaultData()
 {
-#if 1
     // 1: goalie
     // 2: left center back
     // 3(2): right center back
@@ -259,60 +262,32 @@ FormationRBF::createDefaultParam()
     // 11(10): right forward
     createNewRole( 1, "Goalie", Formation::CENTER );
     createNewRole( 2, "CenterBack", Formation::SIDE );
-    createNewRole( 3, "CenterBack", Formation::SYNMETRY );
-    setSynmetryType( 3, 2 );
+    setSymmetryType( 3, 2, "CenterBack" );
     createNewRole( 4, "SideBack", Formation::SIDE );
-    createNewRole( 5, "SideBack", Formation::SYNMETRY );
-    setSynmetryType( 5, 4 );
+    setSymmetryType( 5, 4, "SideBack" );
     createNewRole( 6, "DefensiveHalf", Formation::CENTER );
     createNewRole( 7, "OffensiveHalf", Formation::SIDE );
-    createNewRole( 8, "OffensiveHalf", Formation::SYNMETRY );
-    setSynmetryType( 8, 7 );
+    setSymmetryType( 8, 7, "OffensiveHalf" );
     createNewRole( 9, "SideForward", Formation::SIDE );
-    createNewRole( 10, "SideForward", Formation::SYNMETRY );
-    setSynmetryType( 10, 9 );
+    setSymmetryType( 10, 9, "SideForward" );
     createNewRole( 11, "CenterForward", Formation::CENTER );
 
-    Snapshot snap;
+    SampleData data;
 
-    snap.ball_.assign( 0.0, 0.0 );
-    snap.players_.push_back( Vector2D( -50.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, -8.0 ) );
-    snap.players_.push_back( Vector2D( -20.0, 8.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, -18.0 ) );
-    snap.players_.push_back( Vector2D( -18.0, 18.0 ) );
-    snap.players_.push_back( Vector2D( -15.0, 0.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, -12.0 ) );
-    snap.players_.push_back( Vector2D( 0.0, 12.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, -22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 22.0 ) );
-    snap.players_.push_back( Vector2D( 10.0, 0.0 ) );
+    data.ball_.assign( 0.0, 0.0 );
+    data.players_.push_back( Vector2D( -50.0, 0.0 ) );
+    data.players_.push_back( Vector2D( -20.0, -8.0 ) );
+    data.players_.push_back( Vector2D( -20.0, 8.0 ) );
+    data.players_.push_back( Vector2D( -18.0, -18.0 ) );
+    data.players_.push_back( Vector2D( -18.0, 18.0 ) );
+    data.players_.push_back( Vector2D( -15.0, 0.0 ) );
+    data.players_.push_back( Vector2D( 0.0, -12.0 ) );
+    data.players_.push_back( Vector2D( 0.0, 12.0 ) );
+    data.players_.push_back( Vector2D( 10.0, -22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 22.0 ) );
+    data.players_.push_back( Vector2D( 10.0, 0.0 ) );
 
-    return snap;
-#else
-    // 1: goalie
-    // 2: center back
-    // 3: left side back
-    // 4(3): right side back
-    // 5: left defensive half
-    // 6(5): right defensive half
-    // 7: offensive half
-    // 8: left side half
-    // 9(8): right side half
-    // 10: left forward
-    // 11(10): right forward
-    createNewRole( 1, "Goalie", Formation::CENTER );
-    createNewRole( 2, "CenterBack", Formation::CENTER );
-    createNewRole( 3, "SideBack", Formation::SIDE );
-    setSynmetryType( 4, 3 );
-    createNewRole( 5, "DefensiveHalf", Formation::SIDE );
-    setSynmetryType( 6, 5 );
-    createNewRole( 7, "SideHalf", Formation::SIDE );
-    setSynmetryType( 8, 7 );
-    createNewRole( 9, "OffensiveHalf", Formation::CENTER );
-    createNewRole( 10, "Forward", Formation::SIDE );
-    setSynmetryType( 11, 10 );
-#endif
+    M_samples->addData( *this, data, false );
 }
 
 
@@ -385,7 +360,7 @@ FormationRBF::createNewRole( const int unum,
     }
     else
     {
-        // synmetry
+        // symmetry
     }
 
     // erase old parameter, if exist
@@ -420,8 +395,8 @@ FormationRBF::getPosition( const int unum,
         return Vector2D::INVALIDATED;
     }
     Formation::SideType type = Formation::SIDE;
-    if ( M_synmetry_number[unum - 1] > 0 )  type = Formation::SYNMETRY;
-    if ( M_synmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
+    if ( M_symmetry_number[unum - 1] > 0 )  type = Formation::SYMMETRY;
+    if ( M_symmetry_number[unum - 1] == 0 ) type = Formation::CENTER;
 
     return ptr->getPosition( ball_pos, type );
 }
@@ -447,9 +422,10 @@ FormationRBF::getPositions( const Vector2D & focus_point,
 
  */
 void
-FormationRBF::train( const std::list< Snapshot > & train_data )
+FormationRBF::train()
 {
-    if ( train_data.empty() )
+    if ( ! M_samples
+         || M_samples->dataCont().empty() )
     {
         return;
     }
@@ -475,30 +451,30 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
                   << net.units().size()
                   << std::endl;
 
-        if ( net.units().size() < train_data.size() )
+        if ( net.units().size() < M_samples->dataCont().size() )
         {
-            std::list< Snapshot >::const_iterator snap = train_data.begin();
+            SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
             int count = net.units().size();
             std::cerr << "FormationRBF::train. need to add new center "
-                      << train_data.size() - net.units().size() << std::endl;
+                      << M_samples->dataCont().size() - net.units().size() << std::endl;
             while ( --count >= 0
-                    && snap != train_data.end() )
+                    && d != M_samples->dataCont().end() )
             {
                 std::cerr << "FormationRBF::train. skip known data...." << std::endl;
-                ++snap;
+                ++d;
             }
 
-            while ( snap != train_data.end() )
+            while ( d != M_samples->dataCont().end() )
             {
                 std::cerr << "FormationRBF::train. added new center "
-                          << snap->ball_
+                          << d->ball_
                           << std::endl;
                 RBFNetwork::input_vector center( 2, 0.0 );
-                center[0] = snap->ball_.x;
-                center[1] = snap->ball_.y;
+                center[0] = d->ball_.x;
+                center[1] = d->ball_.y;
                 net.addCenter( center );
 
-                ++snap;
+                ++d;
             }
         }
 
@@ -507,8 +483,7 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
         RBFNetwork::input_vector input( 2, 0.0 );
         RBFNetwork::output_vector teacher( 2, 0.0 );
 
-        const std::list< Formation::Snapshot >::const_iterator snap_end
-            = train_data.end();
+        const SampleDataSet::DataCont::const_iterator d_end = M_samples->dataCont().end();
         int loop = 0;
         double ave_err = 0.0;
         double max_err = 0.0;
@@ -517,16 +492,15 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
         {
             ave_err = 0.0;
             max_err = 0.0;
-            double snap_count = 1.0;
-            for ( std::list< Formation::Snapshot >::const_iterator snap
-                      = train_data.begin();
-                  snap != snap_end;
-                  ++snap, snap_count += 1.0 )
+            double data_count = 1.0;
+            for ( SampleDataSet::DataCont::const_iterator d = M_samples->dataCont().begin();
+                  d != d_end;
+                  ++d, data_count += 1.0 )
             {
-                input[0] = snap->ball_.x;
-                input[1] = snap->ball_.y;
-                teacher[0] = snap->players_[unum - 1].x;
-                teacher[1] = snap->players_[unum - 1].y;
+                input[0] = d->ball_.x;
+                input[1] = d->ball_.y;
+                teacher[0] = d->players_[unum - 1].x;
+                teacher[1] = d->players_[unum - 1].y;
 
                 if ( loop == 2 )
                 {
@@ -542,8 +516,8 @@ FormationRBF::train( const std::list< Snapshot > & train_data )
                     max_err = err;
                 }
                 ave_err
-                    = ave_err * ( ( snap_count - 1.0 ) / snap_count )
-                    + err / snap_count;
+                    = ave_err * ( ( data_count - 1.0 ) / data_count )
+                    + err / data_count;
             }
 
             if ( max_err < 0.001 )
@@ -637,42 +611,13 @@ FormationRBF::param( const int unum ) const
 
  */
 bool
-FormationRBF::read( std::istream & is )
+FormationRBF::readConf( std::istream & is )
 {
-    int n_line = 0;
-
-    n_line += readName( is );
-    if ( n_line < 0 )
-    {
-        return false;
-    }
-
-    //---------------------------------------------------
-    // read each player's parameter set
     if ( ! readPlayers( is ) )
     {
         return false;
     }
 
-    //---------------------------------------------------
-    // check synmetry number circuration reference
-    for ( int i = 0; i < 11; ++i )
-    {
-        int refered_unum = M_synmetry_number[i];
-        if ( refered_unum <= 0 ) continue;
-        if ( M_synmetry_number[refered_unum - 1] > 0 )
-        {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** failed to read formation."
-                      << " Bad synmetrying. player "
-                      << i + 1
-                      << " mirro = " << refered_unum
-                      << " is already synmetrying player"
-                      << std::endl;
-            return false;
-        }
-    }
-
     return true;
 }
 
@@ -697,11 +642,11 @@ FormationRBF::readPlayers( std::istream & is )
 
         // check id
         int unum = 0;
-        int synmetry = 0;
+        int symmetry = 0;
         int n_read = 0;
         if ( std::sscanf( line_buf.c_str(),
                           " player %d %d %n ",
-                          &unum, &synmetry,
+                          &unum, &symmetry,
                           &n_read ) != 2
              || n_read == 0 )
         {
@@ -721,34 +666,34 @@ FormationRBF::readPlayers( std::istream & is )
                       << std::endl;
             return false;
         }
-        if ( synmetry == unum )
+        if ( symmetry == unum )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid synmetry number. at "
+                      << " Invalid symmetry number. at "
                       << i
                       << " mirroing player itself. unum = " << unum
-                      << "  synmetry = " << synmetry
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
-        if ( 11 < synmetry )
+        if ( 11 < symmetry )
         {
             std::cerr << __FILE__ << ":" << __LINE__
                       << " *** ERROR *** failed to read formation."
-                      << " Invalid synmetry number. at "
+                      << " Invalid symmetry number. at "
                       << i
-                      << "  Synmetry number is out of range. unum = " << unum
-                      << "  synmetry = " << synmetry
+                      << "  Symmetry number is out of range. unum = " << unum
+                      << "  symmetry = " << symmetry
                       << std::endl;
             return false;
         }
 
-        M_synmetry_number[i] = synmetry;
+        M_symmetry_number[i] = symmetry;
 
-        // this player is synmetry type
+        // this player is symmetry type
         /*
-          if ( synmetry > 0 )
+          if ( symmetry > 0 )
           {
           ++player_read;
           continue;
@@ -780,36 +725,35 @@ FormationRBF::readPlayers( std::istream & is )
         return false;
     }
 
+    if ( ! std::getline( is, line_buf )
+         || line_buf != "End" )
+    {
+        std::cerr << __FILE__ << ':' << __LINE__ << ':'
+                  << " ***ERROR*** Illegal end tag."
+                  << std::endl;
+        return false;
+    }
+
     return true;
 }
 
 /*-------------------------------------------------------------------*/
 /*!
-  condition <playmode> <>
-  player <unum> 0 <Role> <Param1> <Param2> ...
-  player <unum> <mirro> <Role>
-  ...
-  important <x> <y>
-  important <x> <y>
-  ...
+
 */
 std::ostream &
-FormationRBF::print( std::ostream & os ) const
+FormationRBF::printConf( std::ostream & os ) const
 {
-    printName( os );
-
     for ( int i = 0; i < 11; ++i )
     {
         const int unum = i + 1;
-        os << "player " << unum << " " << M_synmetry_number[i] << " ";
+        os << "player " << unum << " " << M_symmetry_number[i] << '\n';
         /*
-          if ( M_synmetry_number[i] > 0 )
+          if ( M_symmetry_number[i] > 0 )
           {
-          os << '\n';
           continue;
           }
         */
-        os << '\n';
 
         std::map< int, boost::shared_ptr< FormationRBF::Param > >::const_iterator
             it = M_param_map.find( unum );
@@ -831,4 +775,23 @@ FormationRBF::print( std::ostream & os ) const
     return os;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Formation::Ptr
+create()
+{
+    Formation::Ptr ptr( new FormationRBF() );
+    return ptr;
+}
+
+rcss::RegHolder f = Formation::creators().autoReg( &create,
+                                                   FormationRBF::NAME );
+
+}
+
 }
