@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -372,8 +372,8 @@ public:
     double dist2( const Vector2D & p ) const
       {
           //return ( Vector2D( *this ) -= p ).r2();
-          return ( std::pow( this->x - p.x, 2.0 )
-                   + std::pow( this->y - p.y, 2.0 ) );
+          return ( std::pow( this->x - p.x, 2 )
+                   + std::pow( this->y - p.y, 2 ) );
       }
 
     /*!
@@ -455,7 +455,7 @@ public:
       }
 
     /*!
-      \brief rotete this vector with 'deg'
+      \brief rotate this vector with 'deg'
       \param deg rotated angle by double type
       \return reference to itself
      */
@@ -545,11 +545,22 @@ public:
       }
 
     /*!
-      \brief check if this vector is same as given vector
+      \brief check if this vector is strictly same as given vector.
       \param other compared vector
-      \return true if same, otherwise false
+      \return true if strictly same, otherwise false.
     */
     bool equals( const Vector2D & other ) const
+      {
+          return this->x == other.x
+              && this->y == other.y;
+      }
+
+    /*!
+      \brief check if this vector is weakly same as given vector.
+      \param other compared vector.
+      \return true if weakly same, otherwise false.
+    */
+    bool equalsWeakly( const Vector2D & other ) const
       {
           //return dist2( other ) < EPSILON * EPSILON;
           return std::fabs( this->x - other.x ) < EPSILON
@@ -625,7 +636,7 @@ public:
     */
     std::ostream & print( std::ostream & os ) const
       {
-          os << "(" << x << ", " << y << ")";
+          os << '(' << x << ", " << y << ')';
           return os;
       }
 
@@ -638,8 +649,8 @@ public:
     std::ostream & printRound( std::ostream & os,
                                const double & prec = 0.1 ) const
       {
-          os << "("  << rint( x / prec ) * prec
-             << ", " << rint( y / prec ) * prec << ")";
+          os << '('  << rint( x / prec ) * prec
+             << ", " << rint( y / prec ) * prec << ')';
           return os;
       }
 
@@ -653,7 +664,12 @@ public:
     class XCmp
         : public std::binary_function< Vector2D, Vector2D, bool > {
     public:
-        //! functional operator
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
@@ -668,7 +684,12 @@ public:
     class YCmp
         : public std::binary_function< Vector2D, Vector2D, bool > {
     public:
-        //! functional operator
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
@@ -683,7 +704,12 @@ public:
     class AbsXCmp
         : public std::binary_function< Vector2D, Vector2D, bool > {
     public:
-        //! functional operator
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
@@ -698,7 +724,12 @@ public:
     class AbsYCmp
         : public std::binary_function< Vector2D, Vector2D, bool > {
     public:
-        //! functional operator
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
         result_type operator()( const first_argument_type & lhs,
                                 const second_argument_type & rhs ) const
           {
@@ -706,7 +737,33 @@ public:
           }
     };
 
+    /*!
+      \class XYCmp
+      \brief comparison predicate for XY value.
+     */
+    class XYCmp
+        : public std::binary_function< Vector2D, Vector2D, bool > {
+    public:
+        /*!
+          \brief functional operator.
+          \param lhs left hand side argument.
+          \param rhs right hand side argument.
+          \return compared result.
+         */
+        result_type operator()( const first_argument_type & lhs,
+                                const second_argument_type & rhs ) const
+          {
+              return ( lhs.x < rhs.x
+                       ? true
+                       : lhs.x > rhs.x
+                       ? false
+                       : lhs.y < rhs.y );
+              // if ( lhs.x < rhs.x ) return true;
+              // else if ( lhs.x > rhs.x ) return false;
+              // else return lhs.y < rhs.y;
+          }
 
+    };
     //////////////////////////////////////////////
     // functor for region
 
@@ -739,7 +796,7 @@ public:
 ////////////////////////////////////////////////////////
 // comparison operators
 /*!
-  \brief check vectors are same or not.
+  \brief check vectors are strictly same or not.
   \param lhs left hand side parameter
   \param rhs right hand side parameter
   \return true if vectors are same.
@@ -755,7 +812,7 @@ operator==( const rcsc::Vector2D & lhs,
 }
 
 /*!
-  \brief check vectors are different or not.
+  \brief check vectors are strictly different or not.
   \param lhs left hand side parameter
   \param rhs right hand side parameter
   \return true if vectors are not same.
