@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -473,12 +473,55 @@ ActionEffector::makeCommand( std::ostream & to )
     {
         M_command_say->toStr( to );
         incCommandCount( PlayerCommand::SAY );
+    }
+
+    return to;
+}
+
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+ActionEffector::clearAllCommands()
+{
+    if ( M_command_body )
+    {
+        delete M_command_body;
+        M_command_body = static_cast< PlayerBodyCommand * >( 0 );
+    }
+
+    if ( M_command_turn_neck )
+    {
+        delete M_command_turn_neck;
+        M_command_turn_neck = static_cast< PlayerTurnNeckCommand * >( 0 );
+    }
+
+    if ( M_command_change_view )
+    {
+        delete M_command_change_view;
+        M_command_change_view = static_cast< PlayerChangeViewCommand * >( 0 );
+    }
+
+    if ( M_command_pointto )
+    {
+        delete M_command_pointto;
+        M_command_pointto = static_cast< PlayerPointtoCommand * >( 0 );
+    }
+
+    if ( M_command_attentionto )
+    {
+        delete M_command_attentionto;
+        M_command_attentionto = static_cast< PlayerAttentiontoCommand * >( 0 );
+    }
+
+    if ( M_command_say )
+    {
         delete M_command_say;
         M_command_say = static_cast< PlayerSayCommand * >( 0 );
     }
     clearSayMessages();
-
-    return to;
 }
 
 /*-------------------------------------------------------------------*/
@@ -490,7 +533,7 @@ ActionEffector::setKick( const double & power,
                          const AngleDeg & rel_dir )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register kick. power= %.1f, rel_dir= %.1f",
+                   __FILE__" (setKick) register kick. power= %.1f, rel_dir= %.1f",
                   power, rel_dir.degree() );
 
     double command_power = power;
@@ -502,7 +545,7 @@ ActionEffector::setKick( const double & power,
                   << " kick power is over max. com=" << command_power
                   << " > sparam=" << ServerParam::i().maxPower() << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setKick. power over. %.10f",
+                       __FILE__": (setKick) power over. %.10f",
                       command_power);
         command_power = ServerParam::i().maxPower();
     }
@@ -512,10 +555,10 @@ ActionEffector::setKick( const double & power,
         std::cerr << M_agent.world().teamName() << ' '
                   << M_agent.world().self().unum() << ": "
                   << M_agent.world().time()
-                  << " kick power is negative " << command_power
+                  << " negative kick power " << command_power
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setKick. power is negative. %.1f",
+                       __FILE__": (setKick) negative power. %.1f",
                       command_power );
         command_power = 0.0;
     }
@@ -523,7 +566,7 @@ ActionEffector::setKick( const double & power,
     command_power = rint( command_power * 1000.0 ) * 0.001;
 
     dlog.addText( Logger::ACTION,
-                   __FILE__": setKick. Power=%.1f  Dir=%.1f  KickRate=%.4f  Accel=%.2f",
+                   __FILE__": (setKick) Power=%.1f  Dir=%.1f  KickRate=%.4f  Accel=%.2f",
                   command_power, rel_dir.degree(), M_agent.world().self().kickRate(),
                   M_agent.world().self().kickRate() * command_power );
 
@@ -559,7 +602,7 @@ ActionEffector::setKick( const double & power,
     M_kick_accel_error.assign( maxrnd, maxrnd );
 
     dlog.addText( Logger::SYSTEM,
-                   __FILE__": setKick. accel=(%f, %f) err=(%f, %f)",
+                   __FILE__": (setKick) accel=(%f, %f) err=(%f, %f)",
                   M_kick_accel.x, M_kick_accel.y,
                   M_kick_accel_error.x, M_kick_accel_error.y );
 }
@@ -590,10 +633,10 @@ conserve_dash_power( const WorldModel & world,
         std::cerr << world.teamName() << ' '
                   << world.self().unum() << ": "
                   << world.time()
-                  << " dash power should be positive now"
+                  << " (conserve_dash_power) dash power should be positive now"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": dash power should be positive now" );
+                       __FILE__" (conserve_dash_power) dash power should be positive now" );
     }
 
     const double available_stamina
@@ -604,7 +647,7 @@ conserve_dash_power( const WorldModel & world,
     if ( available_stamina < required_stamina )
     {
         dlog.addText( Logger::ACTION,
-                       __FILE__": conserve_dash_power: no stamina. power = %.1f. stamina = %.1f",
+                       __FILE__" (conserve_dash_power) no stamina. power = %.1f. stamina = %.1f",
                       power, available_stamina );
         power = available_stamina;
         if ( back_dash )
@@ -636,7 +679,7 @@ conserve_dash_power( const WorldModel & world,
         power *= -1.0;
     }
     dlog.addText( Logger::ACTION,
-                   __FILE__": conserved power = %.1f",
+                   __FILE__" (conserve_dash_power) conserved power = %.1f",
                   power );
 
     power = param.normalizeDashPower( power );
@@ -667,7 +710,7 @@ ActionEffector::setDash( const double & power,
     const ServerParam & param = ServerParam::i();
 
     dlog.addText( Logger::ACTION,
-                   __FILE__": register dash. power=%.1f, dir=%.1f",
+                   __FILE__" (setDash) register dash. power=%.1f, dir=%.1f",
                   power, rel_dir.degree() );
 
     //
@@ -680,7 +723,7 @@ ActionEffector::setDash( const double & power,
          || param.maxDashPower() + 0.01 < command_power )
     {
         dlog.addText( Logger::ACTION,
-                       __FILE__": over dash power range" );
+                       __FILE__" (setDash) over dash power range" );
         std::cerr << M_agent.world().teamName() << ' '
                   << M_agent.world().self().unum() << ": "
                   << M_agent.world().time()
@@ -699,7 +742,7 @@ ActionEffector::setDash( const double & power,
          || param.maxDashAngle() + 0.01 < command_dir )
     {
         dlog.addText( Logger::ACTION,
-                      __FILE__": over dash angle range %.1f",
+                      __FILE__" (setDash) over dash angle range %.1f",
                       rel_dir.degree() );
         std::cerr << M_agent.world().teamName() << ' '
                   << M_agent.world().self().unum() << ": "
@@ -780,7 +823,7 @@ ActionEffector::setDash( const double & power,
                             accel_mag * ServerParam::i().playerRand() );
 #endif
     dlog.addText( Logger::SYSTEM,
-                   __FILE__": setDash. power=%.3f dir=%.3f, accel=(%.3f, %.3f) r=%.3f th=%.3f",
+                   __FILE__" (setDash) power=%.3f dir=%.3f, accel=(%.3f, %.3f) r=%.3f th=%.3f",
                   command_power, command_dir,
                   M_dash_accel.x, M_dash_accel.y,
                   accel_mag, accel_angle.degree() );
@@ -793,10 +836,6 @@ ActionEffector::setDash( const double & power,
 void
 ActionEffector::setTurn( const AngleDeg & moment )
 {
-    dlog.addText( Logger::ACTION,
-                   __FILE__": register turn. moment = %.1f",
-                  moment.degree() );
-
     const double my_speed = M_agent.world().self().vel().r();
     double command_moment = moment.degree();
     // required turn param
@@ -806,7 +845,7 @@ ActionEffector::setTurn( const AngleDeg & moment )
                  * M_agent.world().self().playerType().inertiaMoment() ) );
 
     dlog.addText( Logger::ACTION,
-                   __FILE__": register turn. moment= %.1f, cmd_param= %.1f, my_inertia= %.1f",
+                   __FILE__" (setTurn) register turn. moment=%.1f, cmd_param=%.1f, my_inertia=%.1f",
                   moment.degree(), command_moment,
                   M_agent.world().self().playerType().inertiaMoment() );
 
@@ -814,14 +853,14 @@ ActionEffector::setTurn( const AngleDeg & moment )
     if ( command_moment > ServerParam::i().maxMoment() )
     {
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurn. over max moment. moment= %.1f, command= %.1f",
+                       __FILE__" (setTurn) over max moment. moment=%.1f, command=%.1f",
                       moment.degree(), command_moment );
         command_moment = ServerParam::i().maxMoment();
     }
     if ( command_moment < ServerParam::i().minMoment() )
     {
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurn. under min moment. moment= %.1f, command= %.1f",
+                       __FILE__" (setTurn) under min moment. moment=%.1f, command=%.1f",
                       moment.degree(), command_moment );
         command_moment = ServerParam::i().minMoment();
     }
@@ -858,7 +897,7 @@ ActionEffector::setTurn( const AngleDeg & moment )
                               * M_turn_actual );
 
     dlog.addText( Logger::SYSTEM,
-                   __FILE__": setTurn. command_moment= %.2f. actual_turn= %.2f. error= %.2f",
+                   __FILE__" (setTurn) command_moment=%.2f. actual_turn=%.2f. error=%.2f",
                   command_moment, M_turn_actual, M_turn_error );
 }
 
@@ -872,7 +911,7 @@ ActionEffector::setMove( const double & x,
 
 {
     dlog.addText( Logger::ACTION,
-                  "register move. (%.1f, %.1f)",
+                  __FILE__" (setMove) register move. (%.1f, %.1f)",
                   x, y );
 
     double command_x = x;
@@ -889,7 +928,7 @@ ActionEffector::setMove( const double & x,
                   << " Must move to a place in the pitch ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setMove. (%.1f, %.1f). must move to a place in pitch",
+                       __FILE__" (setMove) (%.1f, %.1f). must move to a place in pitch",
                       command_x, command_y );
         command_x = min_max( - ServerParam::i().pitchHalfLength(),
                              command_x,
@@ -910,7 +949,7 @@ ActionEffector::setMove( const double & x,
                   << " Must move to a place in our half ("
                   << command_x << ", " << command_y << ")" << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setMove. (%.1f, %.1f). must move to a place in our half",
+                       __FILE__" (setMove) (%.1f, %.1f). must move to a place in our half",
                       command_x, command_y );
         command_x = -0.1;
     }
@@ -928,7 +967,7 @@ ActionEffector::setMove( const double & x,
                       << " Must move to a place within penalty area(1) ("
                       << command_x << ", " << command_y << ")" << std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setMove. (%.1f, %.1f). must move to a place in penalty area(1)",
+                           __FILE__" (setMove) (%.1f, %.1f). must move to a place in penalty area(1)",
                           command_x, command_y );
             command_x = - ServerParam::i().pitchHalfLength() + 1.0;
         }
@@ -940,7 +979,7 @@ ActionEffector::setMove( const double & x,
                       << " Must move to a place within penalty area(2) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setMove. (%.1f, %.1f). must move to a place in penalty area(2)",
+                           __FILE__" (setMove) (%.1f, %.1f). must move to a place in penalty area(2)",
                           command_x, command_y );
             command_x = - ServerParam::i().ourPenaltyAreaLineX() - 1.0;
         }
@@ -952,7 +991,7 @@ ActionEffector::setMove( const double & x,
                       << " Must move to a place within penalty area(3) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setMove. (%.1f, %.1f). must move to a place in penalty area(3)",
+                           __FILE__" (setMove) (%.1f, %.1f). must move to a place in penalty area(3)",
                           command_x, command_y );
             command_y = ServerParam::i().penaltyAreaHalfWidth() - 1.0;
         }
@@ -964,7 +1003,7 @@ ActionEffector::setMove( const double & x,
                       << " Must move to a place within penalty area(4) ("
                       << command_x << ", " << command_y << ")"<< std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setMove. (%.1f, %.1f). must move to a place in penalty area(4)",
+                           __FILE__" (setMove) (%.1f, %.1f). must move to a place in penalty area(4)",
                           command_x, command_y );
             command_y = - ServerParam::i().penaltyAreaHalfWidth() + 1.0;
         }
@@ -990,7 +1029,7 @@ void
 ActionEffector::setCatch()
 {
     dlog.addText( Logger::ACTION,
-                  "register catch" );
+                  __FILE__" (setCatch) register catch" );
 
     // catch area is rectangle.
     // "real" catchable length is the length of diagonal line.
@@ -1001,7 +1040,7 @@ ActionEffector::setCatch()
     // relative angle
     AngleDeg catch_angle = M_agent.world().ball().angleFromSelf() - M_agent.world().self().body();
     dlog.addText( Logger::ACTION,
-                   __FILE__": setCatch. ball_rel_angle= %.1f  diagonal_angle= %.1f",
+                   __FILE__" (setCatch) ball_rel_angle= %.1f  diagonal_angle= %.1f",
                   catch_angle.degree(), diagonal_angle );
 
     // add diagonal angle
@@ -1025,7 +1064,7 @@ void
 ActionEffector::setTackle( const double & power_or_dir )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register tackle. power_or_dir=%.1f",
+                   __FILE__" (setTackle) register tackle. power_or_dir=%.1f",
                   power_or_dir );
 
     double command_arg = power_or_dir;
@@ -1040,7 +1079,7 @@ ActionEffector::setTackle( const double & power_or_dir )
                       << "tackle dir over the range. dir=" << command_arg
                       << std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setTackle. dir over. %f",
+                           __FILE__": (setTackle) dir over. %f",
                           command_arg );
         }
         command_arg = AngleDeg::normalize_angle( power_or_dir );
@@ -1056,7 +1095,7 @@ ActionEffector::setTackle( const double & power_or_dir )
                       << " > sparam=" << ServerParam::i().maxTacklePower()
                       << std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setTackle. power over. %f",
+                           __FILE__": (setTackle) power over. %f",
                           command_arg );
             command_arg = ServerParam::i().maxTacklePower();
         }
@@ -1069,7 +1108,7 @@ ActionEffector::setTackle( const double & power_or_dir )
                       << "tackle power underflow " << command_arg
                       << std::endl;
             dlog.addText( Logger::ACTION,
-                           __FILE__": setKick. power underflow. %f",
+                           __FILE__": (setTackle) power underflow. %f",
                           command_arg );
             command_arg = ServerParam::i().minPower();
         }
@@ -1109,7 +1148,7 @@ void
 ActionEffector::setTurnNeck( const AngleDeg & moment )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register turn_neck. moment= %.1f",
+                   __FILE__" (setTurnNeck) register turn_neck. moment=%.1f",
                   moment.degree() );
 
     double command_moment = moment.degree();
@@ -1120,10 +1159,10 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
         std::cerr << M_agent.world().teamName() << ' '
                   << M_agent.world().self().unum() << ": "
                   << M_agent.world().time()
-                  << "  doTurnNeck. over max moment. "
+                  << " (setTurnNeck) over max moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurnNeck. over max moment. %.1f",
+                       __FILE__" (setTurnNeck) over max moment. %.1f",
                       command_moment );
         command_moment = ServerParam::i().maxNeckMoment();
     }
@@ -1132,10 +1171,10 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
         std::cerr << M_agent.world().teamName() << ' '
                   << M_agent.world().self().unum() << ": "
                   << M_agent.world().time()
-                  << "  doTurnNeck. under min moment. "
+                  << " (setTurnNeck) under min moment. "
                   << command_moment << std::endl;
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurnNeck. under min moment. %.1f",
+                       __FILE__" (setTurnNeck) under min moment. %.1f",
                       command_moment );
         command_moment = ServerParam::i().minNeckMoment();
     }
@@ -1151,7 +1190,7 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
         command_moment = rint( ServerParam::i().maxNeckAngle()
                                - M_agent.world().self().neck().degree() );
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurnNeck. next_neck= %.1f. over max. new-moment= %.1f",
+                       __FILE__" (setTurnNeck) next_neck= %.1f. over max. new-moment= %.1f",
                       next_neck_angle.degree(), command_moment );
     }
 
@@ -1160,7 +1199,7 @@ ActionEffector::setTurnNeck( const AngleDeg & moment )
         command_moment = rint( ServerParam::i().minNeckAngle()
                                - M_agent.world().self().neck().degree() );
         dlog.addText( Logger::ACTION,
-                       __FILE__": setTurnNeck. next_neck= %.1f. under min. new-momment= %.1f",
+                       __FILE__" (setTurnNeck) next_neck= %.1f. under min. new-momment= %.1f",
                       next_neck_angle.degree(), command_moment );
     }
 
@@ -1185,7 +1224,7 @@ void
 ActionEffector::setChangeView( const ViewWidth & width )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register change_view. width= %d",
+                   __FILE__" (setChangeView) register change_view. width= %d",
                   width.type() );
 
     //////////////////////////////////////////////////
@@ -1239,7 +1278,7 @@ void
 ActionEffector::addSayMessage( const SayMessage * message )
 {
     dlog.addText( Logger::ACTION,
-                  __FILE__": add new say message.[%c]",
+                  __FILE__" (addSayMessage) add new say message.[%c]",
                   message->header() );
 
     M_say_messages.push_back( message );
@@ -1253,7 +1292,7 @@ bool
 ActionEffector::removeSayMessage( const char header )
 {
     dlog.addText( Logger::ACTION,
-                  __FILE__": remove say message. header=[%c]", header );
+                  __FILE__" (removeSayMessage) header=[%c]", header );
 
     bool removed = false;
 
@@ -1266,7 +1305,7 @@ ActionEffector::removeSayMessage( const char header )
             it = M_say_messages.erase( it );
             removed = true;
             dlog.addText( Logger::ACTION,
-                          __FILE__": removed" );
+                          __FILE__" (removeSayMessage) removed" );
         }
         else
         {
@@ -1286,7 +1325,7 @@ ActionEffector::setPointto( const double & x,
                             const double & y )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register pointto. (%.2f, %.2f)",
+                   __FILE__" (setPointto) register pointto. (%.2f, %.2f)",
                   x, y );
 
     Vector2D target_pos( x, y );
@@ -1315,7 +1354,7 @@ void
 ActionEffector::setPointtoOff()
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register pointto off" );
+                   __FILE__" (setPointtoOff) register pointto off" );
 
     //////////////////////////////////////////////////
     // create command object
@@ -1339,7 +1378,7 @@ ActionEffector::setAttentionto( const SideID side,
                                 const int unum )
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register attentionto. side= %d, unum= %d",
+                   __FILE__" (setAttentionto) register attentionto. side= %d, unum= %d",
                   side, unum );
 
     //////////////////////////////////////////////////
@@ -1365,7 +1404,7 @@ void
 ActionEffector::setAttentiontoOff()
 {
     dlog.addText( Logger::ACTION,
-                   __FILE__": register attentionto off" );
+                   __FILE__" (setAttentiontoOff) register attentionto off" );
 
     //////////////////////////////////////////////////
     // create command object
@@ -1432,7 +1471,7 @@ ActionEffector::makeSayCommand()
                                           M_agent.config().version() );
 
     dlog.addText( rcsc::Logger::ACTION,
-                  __FILE__": say message [%s]",
+                  __FILE__" (makeSayCommand) say message [%s]",
                   M_say_message.c_str() );
 }
 
@@ -1470,12 +1509,6 @@ ActionEffector::queuedNextSelfBody() const
         double moment = 0.0;
         getTurnInfo( &moment, NULL );
         next_angle += moment;
-//         dlog.addText( Logger::SYSTEM,
-//                        __FILE__": queuedNextSelfBody."
-//                       " cur_body=%.1f + moment=%.1f -> %.1f",
-//                       M_agent.world().self().body().degree(),
-//                       moment,
-//                       next_angle.degree() );
     }
 
     return next_angle;
