@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -186,9 +186,9 @@ FullstateSensor::parseV8( const char * msg )
 
       player : ((p {l|r} <unum> {g|<player_type_id>}) <pos.x> <pos.y>
       <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
-      (<stamina> <effort> <recovery>))
+      (<stamina> <effort> <recovery>[ <capacity>]))
 
-      // after rcssserver-8.03, stamina info has been omit.
+      // after rcssserver-8.03, self stamina info has been omit.
       // and added "arm" info.
       */
 
@@ -205,7 +205,7 @@ FullstateSensor::parseV8( const char * msg )
 
     // stamina info or count info
     while ( *msg != '(' ) ++msg; // skip to next token
-    if ( ! strncmp(msg, "(stamina", 8) )
+    if ( ! std::strncmp( msg, "(stamina", 8 ) )
     {
         while (*msg != ')') ++msg; // ignore stamina info
     }
@@ -244,7 +244,7 @@ FullstateSensor::parseV8( const char * msg )
 
     //((p {l|r} <unum> {g|<player_type_id>}) <pos.x> <pos.y>
     // <vel.x> <vel.y> <body_angle> <neck_angle>[ <point_dist> <point_dir>]
-    // (<stamina> <effort> <recovery>))
+    // (<stamina> <effort> <recovery>[ <capacity>]))
     while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(p"
     while ( *msg != '\0' )
     {
@@ -256,7 +256,7 @@ FullstateSensor::parseV8( const char * msg )
 
         PlayerT player;
 
-        while ( *msg != ' ' ) ++msg;
+        while ( *msg != '\0' && *msg != ' ' ) ++msg;
         ++msg; // skip "p "
         player.side_ = ( *msg == 'l'
                          ? LEFT
@@ -280,7 +280,7 @@ FullstateSensor::parseV8( const char * msg )
             while ( *msg != ')' ) ++msg; // skip " g" or " ID"
         }
 
-        while ( *msg != ' ' ) ++msg; // skip to x pos
+        while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip to x pos
 
         player.pos_.x = std::strtod( msg, &next ); msg = next;
         player.pos_.y = std::strtod( msg, &next ); msg = next;
@@ -294,12 +294,27 @@ FullstateSensor::parseV8( const char * msg )
             player.pointto_dist_ = std::strtod( msg, &next ); msg = next;
             player.pointto_dir_ = std::strtod( msg, &next ); msg = next;
         }
-        while ( *msg != '(' ) ++msg; // skip to "(stamina"
-        while ( *msg != ' ' ) ++msg; // skip to space " "
+        while ( *msg != '\0' && *msg != '(' ) ++msg; // skip to "(stamina"
+        while ( *msg != '\0' && *msg != ' ' ) ++msg; // skip to space " "
         ++msg;
         player.stamina_ = std::strtod( msg, &next ); msg = next;
         player.effort_ = std::strtod( msg, &next ); msg = next;
         player.recovery_ = std::strtod( msg, &next ); msg = next;
+        if ( *msg != ')' )
+        {
+            player.stamina_capacity_ = std::strtod( msg, &next ); msg = next;
+        }
+
+        while ( *msg == ' ' ) ++msg;
+
+        if ( *msg == 'k' )
+        {
+            player.kicked_ = true;
+        }
+        else if ( *msg == 't' )
+        {
+            player.tackle_ = true;
+        }
 
         if ( LEFT == player.side_ )
         {
