@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -256,10 +256,6 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
     const Vector2D & pvel = ( player.seenVelCount() <= player.velCount()
                               ? player.seenVel()
                               : player.vel() );
-    const Vector2D first_pos = player_type.inertiaPoint( ppos, pvel, n_turn );
-
-#if 1
-    //const int vel_count = std::min( player.seenVelCount(), player.velCount() );
 
     Vector2D player_pos = inertia_n_step_point( ppos, pvel,
                                                 n_turn + max_dash,
@@ -312,73 +308,6 @@ PlayerIntercept::canReachAfterDash( const int n_turn,
     }
 
     return false;
-
-#else
-
-    Vector2D tmp_pos = first_pos;
-    Vector2D tmp_vel( 0.0, 0.0 );
-
-    if ( player.velCount() < 4 )
-    {
-        tmp_vel = player.vel();
-    }
-    else
-    {
-        tmp_vel.setPolar( player_type.realSpeedMax() * player_type.playerDecay(),
-                          ( ball_pos - first_pos ).th() );
-    }
-
-    tmp_vel *= std::pow( player_type.playerDecay(), n_turn );
-
-    AngleDeg dash_angle = ( ball_pos - tmp_pos ).th();
-
-    double dash_accel_mag = ( ServerParam::i().maxDashPower()
-                              * player_type.dashRate( player_type.effortMax() ) );
-    const Vector2D dash_accel = Vector2D::polar2vector( dash_accel_mag, dash_angle );
-
-    const int total_dash = n_dash + std::min( 5, pos_count - n_turn );
-
-    for ( int i = 0; i < total_dash; ++i )
-    {
-        tmp_vel += dash_accel;
-        // reach max speed
-        if ( tmp_vel.r() > player_type.realSpeedMax() - 0.001 )
-        {
-            double required_dist
-                = ( ball_pos - tmp_pos ).r() - control_area;
-            if ( required_dist
-                 < player_type.realSpeedMax() * ( total_dash - i ) )
-            {
-#ifdef DEBUG
-                dlog.addText( Logger::INTERCEPT,
-                              "________run over by %d dash... p(%.2f %.2f) dash_dist=%.2f",
-                              total_dash - i,
-                              tmp_pos.x, tmp_pos.y,
-                              required_dist );
-#endif
-                return true;
-            }
-            return false;
-        }
-
-        tmp_pos += tmp_vel;
-        tmp_vel *= player_type.playerDecay();
-    }
-
-    if ( first_pos.dist( tmp_pos )
-         > first_pos.dist( ball_pos ) - control_area - player.distFromSelf() * 0.03 )
-    {
-#ifdef DEBUG
-        dlog.addText( Logger::INTERCEPT,
-                      "______dash finised %d. run over... b(%.2f %.2f) p(%.2f %.2f)",
-                      total_dash,
-                      ball_pos.x, ball_pos.y,
-                      tmp_pos.x, tmp_pos.y );
-#endif
-        return true;
-    }
-#endif
-    return false;
 }
 
 /*-------------------------------------------------------------------*/
