@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -76,6 +76,7 @@ public:
     /*!
       \brief write server param
       \param os reference to the output stream
+      \param param network byte order data
       \return serialization result
     */
     virtual
@@ -85,6 +86,7 @@ public:
     /*!
       \brief write player param
       \param os reference to the output stream
+      \param pparam network byte order data
       \return serialization result
     */
     virtual
@@ -94,6 +96,7 @@ public:
     /*!
       \brief write player type param
       \param os reference to the output stream
+      \param type network byte order data
       \return serialization result
     */
     virtual
@@ -113,7 +116,7 @@ public:
     /*!
       \brief write showinfo_t.
       \param os reference to the output stream
-      \param show2 network byte order data
+      \param show network byte order data
       \return reference to the output stream
      */
     virtual
@@ -123,7 +126,7 @@ public:
     /*!
       \brief write showinfo_t2
       \param os reference to the output stream
-      \param show2 data to be output
+      \param show2 serialized data
       \return reference to the output stream
      */
     virtual
@@ -209,7 +212,7 @@ public:
     /*!
       \brief write ShowInfoT
       \param os reference to the output stream
-      \param show2 network byte order data
+      \param show network byte order data
       \return reference to the output stream
      */
     virtual
