@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -696,5 +696,23 @@ SerializerV3::parseParam( const std::string & from,
     return true;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Serializer::Ptr
+create_v3()
+{
+    Serializer::Ptr ptr( new SerializerV3() );
+    return ptr;
+}
+
+rcss::RegHolder v3 = Serializer::creators().autoReg( &create_v3, REC_VERSION_3 );
+
+}
+
 } // end of namespace
 } // end of namespace
